; $Id: dm_choose_cutdata.pro,v 1.18.2.1 2013/05/10 20:15:56 ymqiu Exp $
;#######################################################################
;
; NAME:
;  dm_choose_cutdata
;
; PURPOSE:
;  choose cut data
;
; CATEGORY:
;  dcs_mslice
;
; AUTHOR:
;  Yiming Qiu
;  NIST Center for Neutron Research
;  100 Bureau Drive, Gaithersburg, MD 20899-6102
;  United States
;  yiming.qiu@nist.gov
;  May, 2013
;
; LICENSE:
;  The software in this file is written by an employee of
;  National Institute of Standards and Technology
;  as part of the DAVE software project.
;
;  The DAVE software package is not subject to copyright protection
;  and is in the public domain. It should be considered as an
;  experimental neutron scattering data reduction, visualization, and
;  analysis system. As such, the authors assume no responsibility
;  whatsoever for its use, and make no guarantees, expressed or
;  implied, about its quality, reliability, or any other
;  characteristic. The use of certain trade names or commercial
;  products does not imply any endorsement of a particular product,
;  nor does it imply that the named product is necessarily the best
;  product for the stated purpose. We would appreciate acknowledgment
;  if the DAVE software is used or if the code in this file is
;  included in another product.
;
;#######################################################################

; parameters:
;   None
; keywords:
;   xdat,ydat,yerr: data to be returned
;   (xy)tit:        label of the data
;   extra_tit:      extra title info
;   extra_legd:     extra legend info
pro dcs_mslice::dm_choose_cutdata,xdat=xdat,ydat=ydat,yerr=yerr,xtit=xtit,ytit=ytit,is_E_t=is_E_t,$
    tran_invalid=tran_invalid,extra_tit=extra_tit,extra_legd=extra_legd,debug=debug
    if keyword_set(debug) then begin
       print,'^^choose cut data...'
       current=systime(/sec)
    endif
    extra_tit  = ''
    extra_legd = ''
    if self.samp_typ eq -1 then begin
       ok_ewid  = 0b
       nitem    = n_elements((*self.dataPtr).label)
       tran_invalid = bytarr(nitem-3)
       
       ;xdat
       self->dm_choose_trajdata,self.cut_x[0]+2,data=xdat,title=xtit,is_E=is_E_x
       if is_E_x then begin
          if ~finite(self.cut_step) then begin
             uniq_E = xdat[uniq(xdat,sort(xdat))]
          endif else begin
             if self.cut_step eq 0 then uniq_E = xdat[uniq(xdat,sort(xdat))]
          endelse
       endif

       ;ydat
       self->dm_choose_trajdata,((self.cut_y[0] eq 0)?-1:(self.cut_y[0]+1)),data=ydat,error=yerr,title=ytit
    
       ndat = n_elements(xdat)
       ;thickness
       for i=0,nitem-4 do begin
           if (total(finite((*self.cutsel_tft)[i,*])) ne 0) then begin
              self->dm_choose_trajdata,(*self.cutsel_t)[i]+2,index=index1,range=reform((*self.cutsel_tft)[i,*]),$
                    empty=empty,debug=debug,unit=unit,label=label
              if empty then begin
                 if n_elements(emptyitem) eq 0 then emptyitem = label $
                 else emptyitem = emptyitem+' and '+label
                 index1 = -1 & tmp = temporary(index1)
                 tran_invalid[i] = 1b
              endif else if n_elements(index1) eq ndat then begin
                 index1 = -1 & tmp = temporary(index1)
              endif else begin
                 if n_elements(index) eq 0 then index = index1 $
                 else begin
                    index = dm_common(index,index1,/no_copy)
                 endelse
              endelse
              if ~tran_invalid[i] then extra_legd = self->range_string(extra_legd,label,(*self.cutsel_tft)[i,*],unit) 
           endif
       endfor
    endif else begin
       is_E_t = bytarr(3)
       tran_invalid = bytarr(3)

       emax = max((*self.projStrPtr).en,min=emin)
       dE   = emax-emin    ;default energy integration width

       ;ewid
       if self.eint_yn and (self.samp_typ ne 2) and (self.cut_y[self.samp_typ] eq 0) then $
          self->dm_choose_trajdata,-2,data=ewid
       ok_ewid = (n_elements(ewid) ne 0)

       ;xdat
       self->dm_choose_trajdata,self.cut_x[self.samp_typ],data=xdat,title=xtit,is_E=is_E_x
       if is_E_x then uniq_E = xdat[*,0,0] 

       ndat = n_elements(xdat)
       
       ;thickness
       cut_t2 = (((self.instrname eq 'macs') and self.extravaxis_yn) or (self.samp_typ eq 1) or ((self.samp_typ eq 2) and self.extravaxis_yn) or (self.samp_typ eq 3))    
       cut_t3 = ((self.samp_typ eq 1 or self.samp_typ eq 3) and self.extravaxis_yn)
       ;t1
       if total(finite(self.cut_t1ran)) ne 0 then begin
          self->dm_choose_trajdata,self.cut_t1[self.samp_typ],index=index1,range=self.cut_t1ran,empty=empty,label=label,is_E=is_E_t1,unit=unit
          is_E_t[0] = is_E_t1
          if empty then begin
             index1 = -1 & tmp = temporary(index1)
             emptyitem = label
             tran_invalid[0] = 1b
          endif else if n_elements(index1) eq ndat then begin
             index1 = -1 & tmp = temporary(index1)
          endif
          if ~tran_invalid[0] then extra_legd = self->range_string(extra_legd,label,self.cut_t1ran,unit)
       endif
       ;t2
       if cut_t2 and (total(finite(self.cut_t2ran)) ne 0) then begin
          self->dm_choose_trajdata,self.cut_t2[self.samp_typ],index=index2,range=self.cut_t2ran,empty=empty,label=label,is_E=is_E_t2,unit=unit
          is_E_t[1] = is_E_t2
          if empty then begin
             index2 = -1 & tmp = temporary(index2)
             if n_elements(emptyitem) eq 0 then emptyitem = label $
             else emptyitem = emptyitem+' and '+label
             tran_invalid[1] = 1b
          endif else if n_elements(index2) eq ndat then begin
             index2 = -1 & tmp = temporary(index2)
          endif
          if ~tran_invalid[1] then extra_legd = self->range_string(extra_legd,label,self.cut_t2ran,unit)
       endif
       ;t3
       if cut_t3 and (total(finite(self.cut_t3ran)) ne 0) then begin
          self->dm_choose_trajdata,self.cut_t3[self.samp_typ],index=index3,range=self.cut_t3ran,empty=empty,label=label,is_E=is_E_t3,unit=unit
          is_E_t[2] = is_E_t3
          if empty then begin
             index3 = -1 & tmp = temporary(index3)
             if n_elements(emptyitem) eq 0 then emptyitem = label $
             else emptyitem = emptyitem+' and '+label
             tran_invalid[2] = 1b
          endif else if n_elements(index3) eq ndat then begin
             index3 = -1 & tmp = temporary(index3)
          endif
          if ~tran_invalid[2] then extra_legd = self->range_string(extra_legd,label,self.cut_t3ran,unit)
       endif
    
       ;figure out dE
       if ok_ewid then begin 
          if is_E_x then dE=(finite(self.cut_step)?(self.cut_step):dE) 
          if is_E_t[0] and (total(finite(self.cut_t1ran)) eq 2) and (~ keyword_set(tran_invalid[0])) then begin
             if self.cut_t1ran[1] gt self.cut_t1ran[0] then dE = self.cut_t1ran[1]-self.cut_t1ran[0]
          endif
          if cut_t2 and is_E_t[1] and (total(finite(self.cut_t2ran)) eq 2) and (~ keyword_set(tran_invalid[1])) then begin
             if self.cut_t2ran[1] gt self.cut_t2ran[0] then dE = self.cut_t2ran[1]-self.cut_t2ran[0]
          endif
          if cut_t3 and is_E_t[2] and (total(finite(self.cut_t3ran)) eq 2) and (~ keyword_set(tran_invalid[2])) then begin
             if self.cut_t3ran[1] gt self.cut_t3ran[0] then dE = self.cut_t3ran[1]-self.cut_t3ran[0]
          endif
       endif   

       if n_elements(index1) ne 0 then index = temporary(index1)
       if n_elements(index2) ne 0 then begin
          if n_elements(index) ne 0 then index = dm_common(index,index2,/no_copy) $
          else index = temporary(index2)
       endif
       if n_elements(index3) ne 0 then begin
          if n_elements(index) ne 0 then index = dm_common(index,index3,/no_copy) $
          else index = temporary(index3)
       endif
       
       ;ydat
       self->dm_choose_trajdata, self.cut_y[self.samp_typ]-1,data=ydat,error=yerr,title=ytit
       
       ;extra title info
       if ((self.samp_typ eq 2) and (self.instrname ne 'wand')) or ((self.samp_typ eq 3) and $
          ((is_E_t[0] and tran_invalid[0]) or (is_E_t[1] and tran_invalid[1]) or (is_E_t[2] and tran_invalid[2]))) then begin
          case self.e_bin[self.samp_typ] of
             0:  extra_tit = ''
             1:  extra_tit = 'Elastic'
             2:  extra_tit = self->range_string(extra_tit,'E',self.e_range,'meV')
          endcase
       endif
       if (self.instrname eq 'macs') and (self.macsintn_typ ne 0) then begin
          extra_tit = 'Detector='+(['SPEC','DIFF','SPEC+DIFF'])[self.macsintn_typ]
       endif
    endelse
    
    ok_err = (n_elements(yerr) ne 0)

    if n_elements(index) ne 0 then begin
       if index[0] ne -1 then begin
          xdat = xdat[index]
          ydat = ydat[index]
          if ok_err  then yerr = yerr[index]
          if ok_ewid then ewid = ewid[index]
       endif else begin
          emptyitem = 'combined thickness'
          tran_invalid[*] = 1b
          extra_legd = ''
       endelse
       index = -1
    endif
    
    if (self.samp_typ ge 0) and (self.instrname eq 'dcs') and (total(self.dcsdetmask[1:3]) ne 0) then begin  ;add mask info for DCS
       if strlen(extra_legd) eq 0 then sep='' else sep=', '
       ind = where(1-self.dcsdetmask[1:3],count)
       if count gt 0 then begin
          extra_legd = extra_legd+sep+(['Lower','Central','Upper'])[ind[0]]
          for i=1,count-1 do extra_legd = extra_legd+'+'+(['Lower','Central','Upper'])[ind[i]]
          extra_legd = extra_legd+' Bank'+(['','s'])[count gt 1]
       endif
    endif
    
    if n_elements(emptyitem) ne 0 then $
    ok = dialog_message(['The specified '+emptyitem+' range covers no data.','The entire possible range is used instead.'],dialog_parent=self.tlb)

    ;xrange
    if total(finite(self.cut_ran)) eq 2 then begin
       if self.cut_ran[1] gt self.cut_ran[0] then begin
          index = where((xdat le self.cut_ran[1]) and (xdat ge self.cut_ran[0]),count)
          if count gt 0 then begin
             xdat = xdat[index]
             ydat = ydat[index]
             if ok_err  then yerr = yerr[index]
             if ok_ewid then ewid = ewid[index]
             index = -1
             xstart = self.cut_ran[0]
          endif
       endif
    endif else begin
       if finite(self.cut_ran[0]) then begin
          index = where(xdat ge self.cut_ran[0],count)
          if count gt 0 then begin
             xdat = xdat[index]
             ydat = ydat[index]
             if ok_err  then yerr = yerr[index]
             if ok_ewid then ewid = ewid[index]
             index = -1
             xstart = self.cut_ran[0]
          endif
       endif
       if finite(self.cut_ran[1]) then begin
          index = where(xdat le self.cut_ran[1],count)
          if count gt 0 then begin
             xdat = xdat[index]
             ydat = ydat[index]
             if ok_err  then yerr = yerr[index]
             if ok_ewid then ewid = ewid[index]
             index = -1
          endif
       endif
    endelse

    if keyword_set(debug) then help,xdat,ydat,ewid

    rebinned = 0b
    checkfinite = self.ftypecheckmask or (strlen(self.userf_str) ne 0)
    ;rebinning according to the steps
    if finite(self.cut_step) then begin
       if (self.cut_step ne 0) then begin
          dm_step_bin,self.cut_step,xdat,ydat=ydat,yerr=yerr,debug=debug,avgsum=self.bin_avgsum,$
             bintr=self.binthreshold,conststep=self.bin_conststep,ewid=ewid,checkfinite=checkfinite,$
             extlib=self.bin_extlib,xstart=xstart
          rebinned = 1b
       endif
    endif
    if ~rebinned and is_E_x then begin  ;rebin if x-axis is E
       xmax = max(xdat,min=xmin)
       index = where((uniq_E ge xmin) and (uniq_E le xmax),count)
       if count ne 0 then uniq_E = uniq_E[index]
       if (self.instrname eq 'macs') or (self.samp_typ eq -1) then uniq_E = round(uniq_E*(500d)-(0.501d),/L64)*0.002
       dm_step_bin,1e-5,xdat,ydat=ydat,yerr=yerr,uniq_val=uniq_E,debug=debug,avgsum=self.bin_avgsum,$
             bintr=self.binthreshold,ewid=ewid,checkfinite=checkfinite,extlib=self.bin_extlib,xstart=xstart
    endif else if checkfinite then begin
       index = where(finite(ydat),count)
       if (count ne n_elements(ydat)) and (count ne 0) then begin
          xdat = xdat[index]
          ydat = ydat[index]
          if ok_err  then yerr = yerr[index]
       endif  
    endif
    
    if ok_ewid then begin
       ydat = ydat*dE
       if ok_err then yerr = yerr*dE
    endif

    if keyword_set(debug) then print,'^^choose cut data finished in ',systime(/sec)-current,' sec.'
end