; $Id: dm_choose_slicedata.pro,v 1.24.2.2 2013/05/10 20:15:56 ymqiu Exp $
;#######################################################################
;
; NAME:
;  dm_choose_slicedata
;
; PURPOSE:
;  choose slice data
;
; CATEGORY:
;  dcs_mslice
;
; AUTHOR:
;  Yiming Qiu
;  NIST Center for Neutron Research
;  100 Bureau Drive, Gaithersburg, MD 20899-6102
;  United States
;  yiming.qiu@nist.gov
;  May, 2013
;
; LICENSE:
;  The software in this file is written by an employee of
;  National Institute of Standards and Technology
;  as part of the DAVE software project.
;
;  The DAVE software package is not subject to copyright protection
;  and is in the public domain. It should be considered as an
;  experimental neutron scattering data reduction, visualization, and
;  analysis system. As such, the authors assume no responsibility
;  whatsoever for its use, and make no guarantees, expressed or
;  implied, about its quality, reliability, or any other
;  characteristic. The use of certain trade names or commercial
;  products does not imply any endorsement of a particular product,
;  nor does it imply that the named product is necessarily the best
;  product for the stated purpose. We would appreciate acknowledgment
;  if the DAVE software is used or if the code in this file is
;  included in another product.
;
;#######################################################################

; parameters:
;   None
; keywords:
;   (xyz)dat:     data to be returned
;   zerr:         error bar of zdat
;   (xy)tit:      label of the data
;   zran:         for z data only, contour plot color bar range
;   zmin:         for z data only
;   extra_tit:    extra title info
pro dcs_mslice::dm_choose_slicedata,xdat=xdat,ydat=ydat,zdat=zdat,zerr=zerr,zran=zran,xtit=xtit,ytit=ytit,ztit=ztit,zmin=zmin,zmax=zmax,$
    pran_invalid=pran_invalid,tran_invalid=tran_invalid,extra_tit=extra_tit,debug=debug
    if keyword_set(debug) then begin
       print,'&&choose slice data...'
       current=systime(/sec)
    endif
    
    ;intensity & error bar
    if arg_present(zerr) then begin
      ok_zerr = 1b
      self->dm_choose_trajdata,-1,data=zdat,error=zerr,title=ztit,debug=debug 
    endif else begin
      ok_zerr = 0b
      self->dm_choose_trajdata,-1,data=zdat,title=ztit,debug=debug 
    endelse
    ndat = n_elements(zdat)
    
    extra_tit = ''
    
    nogrid = [0,0]
    
    ;choose x and y data
    self->dm_choose_trajdata,self.slice_x[0>(self.samp_typ)]+((self.samp_typ eq -1)?2:0),data=xdat,title=xtit,is_E=is_E_x,debug=debug
    self->dm_choose_trajdata,self.slice_y[0>(self.samp_typ)]+((self.samp_typ eq -1)?2:0),data=ydat,title=ytit,is_E=is_E_y,debug=debug
    if is_E_x then begin
       if finite(self.slice_xstep) eq 0 then begin
          nogrid[0] = 1 
       endif else begin
          if self.slice_xstep eq 0 then nogrid[0] = 1 
       endelse
    endif
    if is_E_y then begin
       if finite(self.slice_ystep) eq 0 then begin
          nogrid[1] = 1  
       endif else begin
          if self.slice_ystep eq 0 then nogrid[1] = 1 
       endelse
    endif
       
    if self.samp_typ eq -1 then begin ;general multiple-d data
       ok_ewid = 0b
       ;thickness  
       ndat     = n_elements((*self.dataPtr).data[*,0])
       nitem    = n_elements((*self.dataPtr).label)
       if nitem gt 4 then tran_invalid = bytarr(nitem-4)
       for i=0,nitem-5 do begin
           if (total(finite((*self.slice_tft)[i,*])) ne 0) then begin
              self->dm_choose_trajdata,(*self.slice_t)[i]+2,index=index1,range=reform((*self.slice_tft)[i,*]),$
                    empty=empty,debug=debug,unit=unit,label=label
              if empty then begin
                 if n_elements(emptyitem) eq 0 then $
                    emptyitem = label $
                 else $
                    emptyitem = emptyitem+' and '+label 
                 index1 = -1 & tmp = temporary(index1)
                 tran_invalid[i] = 1b
              endif else if n_elements(index1) eq ndat then begin
                 index1 = -1 & tmp = temporary(index1)
              endif else begin
                 if n_elements(index) eq 0 then index = index1 $
                 else begin
                    index = dm_common(index,index1,/no_copy)
                 endelse
              endelse
           endif
           if ~tran_invalid[i] then extra_tit = self->range_string(extra_tit,label,(*self.slice_tft)[i,*],unit)
       endfor
       if n_elements(index) ne 0 then begin
          if index[0] eq -1 then begin
             emptyitem = 'combined thickness'
             if n_elements(tran_invalid) ne 0 then begin
                tran_invalid[*] = 1b
                extra_tit = ''
             endif
          endif else begin
             xdat = xdat[index]
             ydat = ydat[index]
             zdat = zdat[index]
             if ok_zerr then zerr = zerr[index]
             index = -1
          endelse
       endif
       if is_E_x and nogrid[0] then uniq_E_x = xdat[uniq(xdat,sort(xdat))] 
       if is_E_y and nogrid[1] then uniq_E_y = ydat[uniq(ydat,sort(ydat))] 
    endif else begin     
       ;ewid
       if self.eint_yn and (self.samp_typ ne 2) then $
          self->dm_choose_trajdata,-2,data=ewid
       ok_ewid = (n_elements(ewid) eq ndat)
    
       if is_E_x and nogrid[0] then uniq_E_x = xdat[*,0,0]
       if is_E_y and nogrid[1] then uniq_E_y = ydat[*,0,0]
    
       ;t1 thickness
       t1thick = (((self.instrname eq 'macs') and self.extravaxis_yn) or (self.samp_typ eq 1) or (self.samp_typ eq 3) or ((self.samp_typ eq 2) and self.extravaxis_yn))
       if (total(finite(self.slice_t1ran)) ne 0) and t1thick then begin
          self->dm_choose_trajdata,self.slice_t1[self.samp_typ],index=index1,range=self.slice_t1ran,empty=empty,debug=debug,unit=unit,label=label
          if empty then begin
             emptyitem = label
             index1 = -1 & tmp = temporary(index1)
             pran_invalid = 1b
          endif else if n_elements(index1) eq ndat then begin
             index1 = -1 & tmp = temporary(index1)
          endif
          if ~keyword_set(pran_invalid) then extra_tit = self->range_string(extra_tit,label,self.slice_t1ran,unit)
       endif
  
       ;extra viewing axis t2 thickness
       t2thick = (((self.samp_typ eq 1) or (self.samp_typ eq 3)) and self.extravaxis_yn)
       if (total(finite(self.slice_t2ran)) ne 0) and t2thick then begin
          self->dm_choose_trajdata,self.slice_t2[self.samp_typ],index=index2,range=self.slice_t2ran,empty=empty,debug=debug,unit=unit,label=label
          if empty then begin
             if n_elements(emptyitem) eq 0 then emptyitem = label $
             else emptyitem = emptyitem+' and '+label
             index2 = -1 & tmp = temporary(index2)
             tran_invalid = 1b
          endif else if n_elements(index2) eq ndat then begin
             index2 = -1 & tmp = temporary(index2)
          endif 
          if ~keyword_set(tran_invalid) then extra_tit = self->range_string(extra_tit,label,self.slice_t2ran,unit)
       endif

       n_index1 = n_elements(index1)
       n_index2 = n_elements(index2)
       if (n_index1 ne 0) or (n_index2 ne 0) then begin
          if n_index1 eq 0 then index = temporary(index2) $
          else if n_index2 eq 0 then index=temporary(index1) $
          else index = dm_common(index1,index2,/no_copy)
          if index[0] eq -1 then begin
             emptyitem = 'combined thickness'
             tran_invalid = 1b
             pran_invalid = 1b
             extra_tit = ''
          endif else if n_elements(zdat) ne n_elements(index) then begin
             xdat = xdat[index]
             ydat = ydat[index]
             zdat = zdat[index]
             if ok_zerr then zerr = zerr[index]
             if ok_ewid then ewid = ewid[index]
          endif
          index = -1
       endif
       
       if strlen(extra_tit) eq 0 then sep='' else sep=', '
       if ((self.samp_typ eq 2) and (self.instrname ne 'wand')) or ((self.samp_typ eq 3) and (strlen(extra_tit) eq 0) and (~is_E_x) and (~is_E_y)) then begin
          case self.e_bin[self.samp_typ] of
             0:  extra_tit = extra_tit+''
             1:  extra_tit = extra_tit+sep+'Elastic'
             2:  extra_tit = self->range_string(extra_tit,'E',self.e_range,'meV')
          endcase
       endif
       if strlen(extra_tit) eq 0 then sep='' else sep=', '
       if (self.instrname eq 'dcs') and (total(self.dcsdetmask[1:3]) ne 0) then begin  ;add mask info for DCS
          ind = where(1-self.dcsdetmask[1:3],count)
          if count gt 0 then begin
             extra_tit = extra_tit+sep+(['Lower','Central','Upper'])[ind[0]]
             for i=1,count-1 do extra_tit = extra_tit+'+'+(['Lower','Central','Upper'])[ind[i]]
             extra_tit = extra_tit+' Bank'+(['','s'])[count gt 1]
          endif
       endif
       if (self.instrname eq 'macs') and (self.macsintn_typ ne 0) then begin
          extra_tit = extra_tit+sep+'Detector='+(['SPEC','DIFF','SPEC+DIFF'])[self.macsintn_typ]
       endif
    endelse

    if n_elements(emptyitem) ne 0 then $
       ok = dialog_message(['The specified '+emptyitem+' range covers no data.', $
               'The entire possible range is used instead.'],dialog_parent=self.tlb)

    ;horizontal range and vertical range
    n_xran = total(finite(self.slice_xran))
    n_yran = total(finite(self.slice_yran))
    if n_xran eq 2 then begin
       if self.slice_xran[1] gt self.slice_xran[0] then begin
          index = where((xdat le self.slice_xran[1]) and (xdat ge self.slice_xran[0]),count)
          if count gt 0 then begin
             xdat = xdat[index]
             ydat = ydat[index]
             zdat = zdat[index]
             if ok_ewid then ewid = ewid[index]
             if ok_zerr then zerr = zerr[index]
             index = -1
             xstart = self.slice_xran[0]
          endif
       endif
    endif
    if n_yran eq 2 then begin
       if self.slice_yran[1] gt self.slice_yran[0] then begin
          index  = where((ydat le self.slice_yran[1]) and (ydat ge self.slice_yran[0]),count)
          if count gt 0 then begin
             xdat = xdat[index]
             ydat = ydat[index]
             zdat = zdat[index]
             if ok_ewid then ewid = ewid[index]
             if ok_zerr then zerr = zerr[index]
             index = -1
             ystart = self.slice_yran[0]
          endif
       endif
    endif
    if n_xran lt 2 then begin
       if finite(self.slice_xran[0]) then begin
          index = where(xdat ge self.slice_xran[0],count)
          if count gt 0 then begin
             xdat = xdat[index]
             ydat = ydat[index]
             zdat = zdat[index]
             if ok_ewid then ewid = ewid[index]
             if ok_zerr then zerr = zerr[index]
             index = -1
             xstart = self.slice_xran[0]
          endif
       endif
       if finite(self.slice_xran[1]) then begin
          index = where(xdat le self.slice_xran[1],count)
          if count gt 0 then begin
             xdat = xdat[index]
             ydat = ydat[index]
             zdat = zdat[index]
             if ok_ewid then ewid = ewid[index]
             if ok_zerr then zerr = zerr[index]
             index = -1
          endif
       endif
    endif
    if n_yran lt 2 then begin
       if finite(self.slice_yran[0]) then begin
          index = where(ydat ge self.slice_yran[0],count)
          if count gt 0 then begin
             xdat = xdat[index]
             ydat = ydat[index]
             zdat = zdat[index]
             if ok_ewid then ewid = ewid[index]
             if ok_zerr then zerr = zerr[index]
             index = -1
             ystart = self.slice_yran[0]
          endif
       endif
       if finite(self.slice_yran[1]) then begin
          index = where(ydat le self.slice_yran[1],count)
          if count gt 0 then begin
             xdat = xdat[index]
             ydat = ydat[index]
             zdat = zdat[index]
             if ok_ewid then ewid = ewid[index]
             if ok_zerr then zerr = zerr[index]
             index = -1
          endif
       endif
    endif

    ;rebinning according to the steps
    ;rebin even if hstep and vstep are not specified for faster contour plotting
    xmax  = max(xdat,min=xmin)
    ymax  = max(ydat,min=ymin)
    hstep = (xmax-xmin)/400.
    vstep = (ymax-ymin)/400.
    if nogrid[0] then begin
       index = where((uniq_E_x ge xmin) and (uniq_E_x le xmax),count)
       if count ne 0 then uniq_E_x = uniq_E_x[index]
       if (self.instrname eq 'macs') or (self.samp_typ eq -1) then uniq_E_x = round(uniq_E_x*(500d)-(0.501d),/L64)*0.002
       extra = {uniq_xval:uniq_E_x} 
    endif
    if nogrid[1] then begin
       index = where((uniq_E_y ge ymin) and (uniq_E_y le ymax),count)
       if count ne 0 then uniq_E_y = uniq_E_y[index]
       if (self.instrname eq 'macs') or (self.samp_typ eq -1) then uniq_E_y = round(uniq_E_y*(500d)-(0.501d),/L64)*0.002
       if n_elements(extra) eq 0 then $
          extra = {uniq_yval:uniq_E_y} $
       else $
          extra = create_struct('uniq_yval',uniq_E_y,extra)
    endif
    index = -1
    if finite(self.slice_xstep) then begin
       if (self.slice_xstep ne 0) then begin
          hstep = abs(self.slice_xstep)
       endif
    endif
    if finite(self.slice_ystep) then begin
       if (self.slice_ystep ne 0) then begin
          vstep = abs(self.slice_ystep)
       endif
    endif
    if ok_ewid then begin
       dE = 0
       if is_E_x then dE = hstep
       if is_E_y then dE = vstep
       if ((self.samp_typ eq 1) or (self.samp_typ eq 3)) and (~is_E_x) and (~is_E_y) then begin
          tmp_ran = self.slice_t1ran
          emax = max((*self.projStrPtr).en,min=emin)
          if finite(tmp_ran[0]) eq 0 then tmp_ran[0]=-1e30
          if finite(tmp_ran[1]) eq 0 then tmp_ran[1]=1e30
          if tmp_ran[0] lt emin then tmp_ran[0]=emin
          if tmp_ran[1] gt emax then tmp_ran[1]=emax
          if tmp_ran[1] gt tmp_ran[0] then dE=tmp_ran[1]-tmp_ran[0] $
          else dE=emax-emin
       endif
       if dE le 0 then dE=1.0
    endif

    checkfinite = self.ftypecheckmask or (strlen(self.userf_str) ne 0)
    dm_stepgrid_bin,hstep,vstep,xdat,ydat,zdat,zerr=zerr,avgsum=self.bin_avgsum,bintr=self.binthreshold,xstart=xstart,ystart=ystart,$
        conststep=self.bin_conststep,ewid=ewid,debug=debug,checkfinite=checkfinite,extlib=self.bin_extlib,_extra=extra

    if ok_ewid then zdat=zdat*dE

    ;smooth regular grid data
    index  = where(finite(zdat,/nan),count)     ;bins with no data
    refill = (count ne 0)                   ;flag for refilling
    if self.slice_smooth gt 0 then begin     ;smooth data 
       tmp = size(zdat,/dimension)
       if (n_elements(tmp) gt 1) and (min(tmp) gt 3) then begin
          if ok_zerr then zerr = zerr^2
          if self.idl_version ge 6.2 then begin
             for i=0, self.slice_smooth-1 do begin
                 zdat = convol(zdat,self.slice_sm_kernel,/edge_truncate,/nan,/normalize)
                 if ok_zerr then zerr = convol(zerr,self.slice_sm_kernel,/edge_truncate,/nan,/normalize)
                 if refill then begin
                    zdat[index] = !values.f_nan
                    if ok_zerr then zerr[index] = !values.f_nan
                 endif
             endfor
          endif else begin
             for i=0, self.slice_smooth-1 do begin
                 ;put 0 in bins with no data
                 if refill then begin
                    zdat[index] = 0.0
                    if ok_zerr then zerr[index] = 0.0
                 endif
                 zdat = convol(zdat,self.slice_sm_kernel,total(self.slice_sm_kernel),/edge_truncate)
                 if ok_zerr  then zerr = convol(zerr,self.slice_sm_kernel,total(self.slice_sm_kernel),/edge_truncate)
             endfor
             if refill then begin
                zdat[index] = !values.f_nan
                if ok_zerr then zerr[index] = !values.f_nan
             endif
          endelse
          if ok_zerr then zerr = sqrt(zerr)
       endif
    endif
    index = -1
    ;intensity range
    oldzmax  = max(zdat,min=oldzmin,/nan)
    zmax = oldzmax & zmin = oldzmin
    if finite(self.slice_iran[0]) then $
       if self.slice_iran[0] lt oldzmax then zmin=self.slice_iran[0]
    if finite(self.slice_iran[1]) then $
       if self.slice_iran[1] gt oldzmin then zmax=self.slice_iran[1]
    zran = [zmin,zmax]
    if keyword_set(debug) then begin
       help,xdat,nogrid,uniq_E_x,ydat,uniq_E_y
       print,'&&choose slice data finished in ',systime(/sec)-current,' sec.'
    endif
end