; $Id: dm_filetools.pro,v 1.53.2.4 2013/05/29 15:09:25 ymqiu Exp $
;#######################################################################
;
; NAME:
;  dm_filetools
;
; PURPOSE:
;  handle file type menu and file tool menu events and all data file loading related events
;
; CATEGORY:
;  dcs_mslice
;
; HISTORY:
;  02/2010: added NXSPE support - Andrei Savici (saviciat@ornl.gov)   
; 
; AUTHOR:
;  Yiming Qiu
;  NIST Center for Neutron Research
;  100 Bureau Drive, Gaithersburg, MD 20899-6102
;  United States
;  yiming.qiu@nist.gov
;  May, 2013
;
; LICENSE:
;  The software in this file is written by an employee of
;  National Institute of Standards and Technology
;  as part of the DAVE software project.
;
;  The DAVE software package is not subject to copyright protection
;  and is in the public domain. It should be considered as an
;  experimental neutron scattering data reduction, visualization, and
;  analysis system. As such, the authors assume no responsibility
;  whatsoever for its use, and make no guarantees, expressed or
;  implied, about its quality, reliability, or any other
;  characteristic. The use of certain trade names or commercial
;  products does not imply any endorsement of a particular product,
;  nor does it imply that the named product is necessarily the best
;  product for the stated purpose. We would appreciate acknowledgment
;  if the DAVE software is used or if the code in this file is
;  included in another product.
;
;#######################################################################

; parameter:
;   eventname: event name = widget_info(event.id,/uname)
; keyword:
;   event:     passed event structure

;files need to be included
@dm_load_dcsdata
@dm_load_macs
@dm_load_spe
@dm_load_nxspe
@dm_load_phx
@dm_load_inx
@dm_load_wand

;combine data with same rotation angles
;parameters:
;   ang:         [nang] rotation angle
;   qty,dqty:    [*,nang] or [*,*,nang] data
;   weight:      [nang] weight of the data, the operation is sum
;keywords:
;   extra:       [nang,*], optional extra data to be combined, the operation is averaging
;   checkfinite: flag for checking finite values of qty
;   resolution:  a natural number indicating the number of decimal places for resolution, default 2
pro dm_combine_psi,ang,qty,dqty,weight,extra=extra,checkfinite=checkfinite,resolution=resolution
    if n_elements(ang) lt 2 then return   ;no action required
    if n_elements(resolution) eq 0 then resolution = 2 else resolution = 1>(fix(resolution))
    type = size(ang,/type)
    if type ne size('a',/type) then newang = round(ang*((10d)^resolution)+(0.0001d),/L64)/((10d)^resolution) else newang = ang
    ind  = sort(newang) & nang = n_elements(newang) 
    uind = uniq(newang[ind]) & nun = n_elements(uind)
    if n_elements(extra) ne 0 then check_extra = (n_elements(extra[*,0]) eq nang) else check_extra = 0b
    if nun ne nang then begin
       ndim   = size(qty,/n_dimensions)
       newang = newang[ind]
       weight = weight[ind]
       if check_extra  then extra = extra[ind,*]
       if ndim eq 2 then begin      ;single E
          qty  = qty[*,ind]
          dqty = dqty[*,ind]
       endif else begin             ;multiple E
          qty  = qty[*,*,ind]
          dqty = dqty[*,*,ind]
       endelse
       dqty = dqty^2                ;error bar
       j = 0L
       for i=0L,nun-1 do begin
           qtmp = 0.0
           etmp = 0.0
           wgt  = 0.0
           cnt  = 0.0
           if check_extra then ext = 0.0
           while(newang[j] eq newang[uind[i]]) do begin
              cnt = cnt+1.0
              if ndim eq 2 then begin     ;single E
                 if keyword_set(checkfinite) then begin
                    mask = where(finite(qty[*,j],/nan),mskcount)
                    ind  = where(finite(qtmp[*],/nan),count)
                    if count gt 0 then begin
                       qtmp[ind] = qty[ind,j]*wgt
                       etmp[ind] = dqty[ind,j]*((wgt+weight[j])^2-weight[j]^2)
                    endif
                    if (mskcount gt 0) and (wgt ne 0) then begin
                       qty[mask,j]  = qtmp[mask]/wgt
                       dqty[mask,j] = etmp[mask]*((wgt+weight[j])^2-wgt^2)/(wgt^2*weight[j]^2) 
                    endif
                 endif
                 qtmp = temporary(qtmp)+qty[*,j]*weight[j]
                 etmp = temporary(etmp)+dqty[*,j]*(weight[j]^2)
              endif else begin            ;multiple E
                 if keyword_set(checkfinite) then begin
                    ii = 0L
                    while(total(finite(qty[ii,*,j])) eq 0) do ii=ii+1   ;in case the first time channel is masked
                    mask = where(finite(qty[ii,*,j],/nan),mskcount)
                    ii = 0L
                    while(total(finite(qtmp[ii,*])) eq 0) do ii=ii+1    ;in case the first time channel is masked
                    ind = where(finite(qtmp[ii,*],/nan),count) 
                    if count gt 0 then begin
                       qtmp[*,ind] = qty[*,ind,j]*wgt
                       etmp[*,ind] = dqty[*,ind,j]*((wgt+weight[j])^2-weight[j]^2)
                    endif
                    if (mskcount gt 0) and (wgt ne 0) then begin
                       qty[*,mask,j]  = qtmp[*,mask]/wgt
                       dqty[*,mask,j] = etmp[*,mask]*((wgt+weight[j])^2-wgt^2)/(wgt^2*weight[j]^2) 
                    endif
                 endif 
                 qtmp = temporary(qtmp)+qty[*,*,j]*weight[j]
                 etmp = temporary(etmp)+dqty[*,*,j]*(weight[j]^2)
              endelse
              wgt = wgt+weight[j]
              if check_extra then ext = ext+extra[j,*]
              j   = j+1L
              if j eq nang then break
           endwhile
           if wgt eq 0 then wgt=1.0
           weight[i] = wgt
           if check_extra then extra[i,*] = ext/(1.0>cnt)
           if ndim eq 2 then begin    ;single E
              qty[*,i]  = qtmp/wgt
              dqty[*,i] = sqrt(etmp)/wgt
           endif else begin           ;multiple E
              qty[*,*,i]  = qtmp/wgt
              dqty[*,*,i] = sqrt(etmp)/wgt
           endelse
       endfor
       qtmp = 0 & etmp = 0  ;free up the memory
       if ndim eq 2 then begin        ;single E
          qty  = qty[*,0:nun-1]
          dqty = dqty[*,0:nun-1]
       endif else begin               ;multiple E
          qty  = qty[*,*,0:nun-1]
          dqty = dqty[*,*,0:nun-1]
       endelse
       newang = newang[uind]
       weight = weight[0:nun-1]
       if check_extra then extra = extra[0:nun-1,*]
    endif
    if type ne size('a',/type) then ang = fix(newang,type=type) $
    else ang = newang
end

;combine macs data with same A3, Kidney, and energy, data have not been monitor normalized
;parameters:
;   qty,dqty:    [2,ndet,ndat]
;   weight:      [ndat] weight of the data
;   cfx:         [ndat] flag of cfx status
;keywords:
;   aresolution:  a natural number indicating the number of decimal places for angular resolution, default 2
;   eresolution:  a natural number indicating the number of decimal places for energy resolution, default 3
;   viewt:        if set, temperature is regarded as a one of the viewing axis, default is not 
pro dm_combine_macs,qty,dqty,weight,cfx,A3,kidney,Ei,Ef,temperature=temperature,eresolution=eresolution,aresolution=aresolution,viewt=viewt
    ndat = n_elements(weight)
    if (ndat eq 0) or (n_elements(cfx) ne ndat) or (n_elements(A3) ne ndat) or (n_elements(kidney) ne ndat) or (n_elements(Ei) ne ndat) or (n_elements(Ef) ne ndat) then return
    if n_elements(aresolution) eq 0 then aresolution = 2 else aresolution = 1>(fix(aresolution))
    if n_elements(eresolution) eq 0 then eresolution = 3 else eresolution = 1>(fix(eresolution))
    tmp = dm_to_string(cfx)+'_'+dm_to_string(Ei,resolution=eresolution)+'_'+dm_to_string(Ef,resolution=eresolution)+'_'+dm_to_string(A3,resolution=aresolution)+'_'+dm_to_string(kidney,resolution=aresolution)
    if keyword_set(viewt) and (n_elements(temperature) eq ndat*3) then begin
       for i=0L,ndat-1 do tmp[i] = tmp[i]+'_'+dm_to_string(temperature[i,*],resolution=2,separator='_')
       tyn = 1b
    endif else tyn = 0b
    tmpqty  = double(qty) 
    tmpdqty = double(dqty)
    for i=0L,ndat-1 do begin
        tmpqty[*,*,i]  = tmpqty[*,*,i]/(1>weight[i])
        tmpdqty[*,*,i] = tmpdqty[*,*,i]/(1>weight[i]) 
    endfor
    if tyn then $
       dm_combine_psi,tmp,tmpqty,tmpdqty,weight,resolution=aresolution $
    else $
       dm_combine_psi,tmp,tmpqty,tmpdqty,weight,extra=temperature,resolution=aresolution
    ndat1 = n_elements(weight)
    if ndat1 ne ndat then begin
       cfx = intarr(ndat1) & A3 = fltarr(ndat1) & kidney = fltarr(ndat1) & Ei = fltarr(ndat1) & Ef = fltarr(ndat1)
       if tyn then temperature = fltarr(ndat1,3)
       for i=0L,ndat1-1 do begin
           tmpqty[*,*,i]  = tmpqty[*,*,i]*weight[i]
           tmpdqty[*,*,i] = tmpdqty[*,*,i]*weight[i] 
           tmp1 = dm_to_number(strsplit(tmp[i],'_',/extract,count=count))
           if count eq 5+tyn*3 then begin
              cfx[i]    = round(tmp1[0])
              Ei[i]     = tmp1[1]
              Ef[i]     = tmp1[2]
              A3[i]     = tmp1[3]
              kidney[i] = tmp1[4]
              if tyn then temperature[i,*] = tmp1[5:7]
           endif
       endfor
       if size(qty,/type) eq size(1.0,/type) then begin
          qty  = float(tmpqty)
          dqty = float(tmpdqty) 
       endif else begin
          qty  = tmpqty
          dqty = tmpdqty  
       endelse
    endif   
end

;returns start,end,step value
;tolerance is used to determine the step, default is 0.01
;if no step is available then step=!values.f_nan
function dm_startend,value,resolution=resolution,tolerance=tolerance
    n_val = n_elements(value) 
    if n_val eq 0 then return,[!values.f_nan,!values.f_nan,!values.f_nan]
    if n_elements(resolution) eq 0 then resolution = 2
    if n_elements(tolerance) eq 0 then tolerance = 2.0/((10d)^resolution)
    val = round(value*((10d)^resolution)+(0.0001d),/L64)/((10d)^resolution)
    if n_val gt 1 then begin
       val = val[uniq(val,sort(val))] 
       n_val = n_elements(val)
    endif 
    minv = min(val,max=maxv)
    step = !values.f_nan
    avgv = mean(val)
    if max(abs(val-avgv)) gt tolerance then begin
       if n_val ge 2 then begin
          step = (val[n_val-1]-val[0])/(n_val-1.0)
          tmpv = val[0]+findgen(n_val)*step
          if max(abs(tmpv-val)) gt tolerance then step = !values.f_nan $
          else step = dm_to_number(dm_to_string(step,resolution=resolution))
       endif
    endif else step = 0.0
    if step eq 0 then begin
       minv = avgv
       maxv = avgv
    endif
    minv = dm_to_number(dm_to_string(minv,resolution=resolution))
    maxv = dm_to_number(dm_to_string(maxv,resolution=resolution))
    return,[minv,maxv,step]
end

function dcs_mslice::dm_filetools,eventname,combine=combine,event=event,init=init,secondset=secondset,scriptmode=scriptmode,$
    eief=eief,filename=filename,directory=directory,pdt_reset=pdt_reset
    handled = 1b    ;event handled flag
    if keyword_set(scriptmode) then event = {WIDGET_BUTTON, ID:0L, TOP:self.tlb, HANDLER:self.tlb,SELECT:1}
    if strmid(eventname,0,6) eq 'ftype_' then begin ;file type selection event
       type = dm_to_number(strmid(eventname,6,strlen(eventname)-6),/int)
       eventname = 'ftype_'
    endif
    
    case strlowcase(eventname) of
        'emptbut':begin  ;load empty can file(s)
            self.filesel->getproperty,path=open_path,file=open_file,ftpobj=ftpobj
            if keyword_set(scriptmode) and n_elements(filename) ne 0  then open_file=filename
            if keyword_set(scriptmode) and n_elements(directory) ne 0 then open_path=directory
            if n_elements(open_file) eq 0 then begin
               ok = dialog_message(dialog_parent=self.tlb,'Please select empty can files first.',/error,/center)
               break
            endif
            case self.instrname of
              'dcs':begin     ;dcs data
                    self->dm_load_dcs_emptycan,open_path,open_file,parent=event.handler,error=error,$
                        comment=comment,qty=qty,dqty=dqty,ang=ang,weight=weight,adddata=ptr_valid(self.dataStrPtr),ftpobj=ftpobj
                    end
              'macs':begin
                    self->dm_load_macs,open_path,open_file,a3=a3,deteff=deteff,ei=ei,ef=ef,error=error,info=info,cfx=cfx,comment=comment,$
                        kidney=kidney,parent=event.handler,qty=qty,dqty=dqty,title=title,temperature=temperature,weight=weight,ftpobj=ftpobj
                    end
              'spe':begin     ;spe file
                    e_range = self.e_range
                    self->dm_load_spe,open_path,open_file,parent=event.handler,error=error,weight=weight,$
                        qty=qty,dqty=dqty,ang=ang,t_chan=self.e_bin[self.samp_typ],e_range=e_range,$
                        eint_yn=self.eint_yn,avgsum=self.bin_avgsum,diffuse=(self.samp_typ ge 2),$
                        title='loading empty can file...',diffetyp=(self.samp_typ eq 3),$
                        adddata=ptr_valid(self.dataStrPtr),scriptmode=scriptmode
                    if (self.samp_typ ge 2) and (n_elements(e_range) ne 0) and (~ptr_valid(self.dataStrPtr)) then self.e_range=e_range
                    end
              'nxspe':begin   ;nxspe file
                    e_range = self.e_range
                    self->dm_load_nxspe,open_path,open_file,parent=event.handler,error=error,weight=weight,$
                        qty=qty,dqty=dqty,ang=ang,t_chan=self.e_bin[self.samp_typ],e_range=e_range,$
                        eint_yn=self.eint_yn,avgsum=self.bin_avgsum,diffuse=(self.samp_typ ge 2),$
                        title='loading empty can file...',diffetyp=(self.samp_typ eq 3),$
                        adddata=ptr_valid(self.dataStrPtr),scriptmode=scriptmode
                    if (self.samp_typ ge 2) and (n_elements(e_range) ne 0) and (~ptr_valid(self.dataStrPtr)) then self.e_range=e_range
                    end
              'inx':begin     ;inx file
                    e_range = self.e_range
                    self->dm_load_inx,open_path,open_file,parent=event.handler,error=error,weight=weight,$
                        qty=qty,dqty=dqty,ang=ang,t_chan=self.e_bin[self.samp_typ],e_range=e_range,$
                        eint_yn=self.eint_yn,avgsum=self.bin_avgsum,diffuse=(self.samp_typ ge 2),$
                        title='loading empty can file...',diffetyp=(self.samp_typ eq 3),comment=comment,$
                        adddata=ptr_valid(self.dataStrPtr),scriptmode=scriptmode
                    if (self.samp_typ ge 2) and (n_elements(e_range) ne 0) and (~ptr_valid(self.dataStrPtr)) then self.e_range=e_range
                    end
              'wand':begin    ;wand file
                    self->dm_load_wand,open_path,open_file,ang=ang,error=error,parent=event.handler,qty=qty,dqty=dqty,temperature=temperature,$
                        weight=weight,scriptmode=scriptmode
                    end   
                 else: error = 1b ;do nothing
            endcase
            if keyword_set(error) or (n_elements(qty) eq 0) then break
            if keyword_set(secondset) then begin
               ptr_free,self.bgdata2 
            endif else begin
               ptr_free,self.bgdata
            endelse
            if self.instrname eq 'macs' then begin
               dm_combine_macs,qty,dqty,weight,cfx,A3,kidney,Ei,Ef,eresolution=2
               if keyword_set(secondset) then begin
                  self.bgdata2 = ptr_new({qty:temporary(qty),err:temporary(dqty),rotation:temporary(A3),kidney:temporary(kidney),weight:temporary(weight),cfx:temporary(cfx),Ei:temporary(Ei),Ef:temporary(Ef)},/no_copy)
                  if self.extraempcan_yn then self->my_widget_control,'emfcBut',/sensitive
               endif else begin
                  self.bgdata = ptr_new({qty:temporary(qty),err:temporary(dqty),rotation:temporary(A3),kidney:temporary(kidney),weight:temporary(weight),cfx:temporary(cfx),Ei:temporary(Ei),Ef:temporary(Ef)},/no_copy)
                  if self.extraempcan_yn then self->my_widget_control,'emptBut2',/sensitive
               endelse
            endif else begin
               if n_elements(ang) eq 0 then ang=0
               if (self.samp_typ eq 3) and (self.e_bin[4] gt 1) then begin
                  tmp = size(qty,/dimension)
                  dm_grid_ebin,self.e_bin[4],qty,dqty,fltarr(tmp[0]),avgsum=self.bin_avgsum
               endif
               if n_elements(weight) ne n_elements(ang) then weight = fltarr(n_elements(ang))+1.0
               ;combine data with the same angles
               dm_combine_psi,ang,qty,dqty,weight,checkfinite=self.ftypecheckmask
               weight = 0
               if keyword_set(secondset) then begin
                  self.bgdata2 = ptr_new({qty:temporary(qty),err:temporary(dqty),rotation:temporary(ang)},/no_copy)
                  if self.extraempcan_yn then self->my_widget_control,'emfcBut',/sensitive
               endif else begin
                  self.bgdata = ptr_new({qty:temporary(qty),err:temporary(dqty),rotation:temporary(ang)},/no_copy)
                  if self.extraempcan_yn then self->my_widget_control,'emptBut2',/sensitive
               endelse
            endelse
            if ptr_valid(self.bgdata) and ptr_valid(self.bgdata2) then begin
               repeat begin
                  tmp = dm_dialog_input(['empty can 1 factor:','empty can 2 factor:'],/float,default=self.empcanfactor,$
                        info=['I=I_samp-ssfac*(factor1*I_can1+factor2*I_can2)','ssfac: self-shielding factor'],dialog_parent=self.tlb)
                  if total(finite(tmp)) eq 2 then begin
                     self.empcanfactor = tmp
                     ok2 = 1b
                  endif
               endrep until keyword_set(ok2)
            endif
            if n_elements(comment) eq 0 then comment=(*self.ftypename)[self.ftype]+' empty can data '+open_file[0]+(n_elements(open_file) gt 1?'...':'')
            self->set_filehistory,comment,open_path,open_file,ftpobj=ftpobj
            self->my_widget_control,'bgclBut',/sensitive
            if self.idl_version ge 5.6 then self->my_widget_control,'clearDat',tooltip='Background data will not be cleared.'
          end
        'emptbut2':begin
            handled = self->dm_filetools('emptbut',event=event,/secondset,scriptmode=scriptmode,filename=filename,directory=directory)
          end
        'bgflbut':begin  ;load detector dependent background file(s)
            self.filesel->getproperty,path=open_path,file=open_file,ftpobj=ftpobj
            if n_elements(open_file) eq 0 then begin
               ok = dialog_message(dialog_parent=self.tlb,'Please select background files first.',/error,/center)
               break
            endif
            ;quest for the energy range
            default = dm_to_string(self.bgeran)
            repeat begin
                new = dm_dialog_input(['from:','to:'],title='Energy range',xsize=100,default=default,/float,dialog_parent=self.tlb,info='In units of meV')
                if total(finite(new)) eq 2 then begin
                   if new[1] gt new[0] then begin
                      self.bgeran = new
                      ok2 = 1b
                   endif
                endif
            endrep until keyword_set(ok2)
            case self.instrname of
              'dcs':begin     ;dcs data
                    self->dm_load_dcs_detbackground,open_path,open_file,parent=event.handler,error=error,comment=comment,qty=qty,dqty=dqty,ftpobj=ftpobj
                    end
              'spe':begin     ;spe file
                    self->dm_load_spe_detbackground,open_path,open_file,parent=event.handler,error=error,qty=qty,dqty=dqty
                    end
              'nxspe':begin   ;nxspe file
                    self->dm_load_nxspe_detbackground,open_path,open_file,parent=event.handler,error=error,qty=qty,dqty=dqty
                    end   
              'inx':begin     ;inx file
                    self->dm_load_inx_detbackground,open_path,open_file,parent=event.handler,error=error,qty=qty,dqty=dqty,comment=comment
                    end
              else: error = 1b ;do nothing
            endcase
            if keyword_set(error) then break
            if n_elements(qty) ne 0 then begin ;background file
               ptr_free,self.bgdetsPtr
               self.bgdetsPtr = ptr_new(qty)
               if n_elements(comment) eq 0 then comment=(*self.ftypename)[self.ftype]+' det. background data'
               self->set_filehistory,comment,open_path,open_file,ftpobj=ftpobj
               self->my_widget_control,'bgclBut',/sensitive
               if self.idl_version ge 5.6 then self->my_widget_control,'clearDat',tooltip='Background data will not be cleared.'
            endif
          end
        'bgtcbut':begin  ;load time channel background file(s)
            self.filesel->getproperty,path=open_path,file=open_file,ftpobj=ftpobj
            if n_elements(open_file) eq 0 then begin
               ok = dialog_message(dialog_parent=self.tlb,'Please select background files first.',/error,/center)
               break
            endif
            default = self.dcor_eqran & ok = 1b
            eqran   = dm_dialog_input(['E_start (meV):','E_end (meV):','2theta_start:','2theta_end:'],$
                        title='Specify E, 2theta range:',xsize=80,default=default,$
                        info=['The averaged intensity in the specified two theta range',$
                              'will be used as the background for the time channels in',$
                              'the specified E range.'],dialog_parent=self.tlb)
            message = 'Invalid input.'
            if total(finite(eqran)) ne 4 then $
               ok = 0b $
            else if (eqran[0] ge eqran[1]) or (eqran[2] ge eqran[3]) then $
               ok = 0b $
            else if ptr_valid(self.detPosPtr) then begin
               det_tt = (*self.detPosPtr).two_theta
               index  = where((abs(det_tt) ge eqran[2]) and (abs(det_tt) le eqran[3]),count)
               if count eq 0 then begin
                  ok = 0b
                  message = 'Two theta range is incompatible with the detector angles.'
               endif
            endif
            if (~ ok) then begin
               message = [message,'The background file'+$
                        ((n_elements(open_file) gt 1)?'s are':' is')+' not loaded.']
               ok = dialog_message(message,dialog_parent=self.tlb,/center)
               break
            endif
            self.dcor_eqran = eqran
            case self.instrname of
              'dcs':begin     ;dcs data
                    self->dm_load_dcs_tchanbackground,open_path,open_file,parent=event.handler,error=error,comment=comment,qty=qty,dqty=dqty,emean=emean,ftpobj=ftpobj
                    end
              'spe':begin     ;spe file
                    self->dm_load_spe_tchanbackground,open_path,open_file,parent=event.handler,error=error,qty=qty,dqty=dqty,emean=emean
                    end
              'nxspe':begin   ;nxspe file
                    self->dm_load_nxspe_tchanbackground,open_path,open_file,parent=event.handler,error=error,qty=qty,dqty=dqty,emean=emean
                    end   
              'inx':begin     ;inx file
                    self->dm_load_inx_tchanbackground,open_path,open_file,parent=event.handler,error=error,qty=qty,dqty=dqty,comment=comment,emean=emean
                    end
              else: error = 1b ;do nothing
            endcase
            if keyword_set(error) then break
            if (~ ptr_valid(self.detPosPtr)) then begin
               ok = dialog_message('The detector information is missing. Background data is discarded.',dialog_parent=self.tlb,/center)
               break
            endif
            if n_elements(qty) ne 0 then begin ;background file
               if (self.samp_typ eq 3) and (self.e_bin[4] gt 1) then begin
                  dm_grid_ebin,self.e_bin[4],qty,dqty,emean,avgsum=self.bin_avgsum
               endif
               det_tt = (*self.detPosPtr).two_theta
               ;for SPE and NXSPE file type, check masks
               if self.ftypecheckmask then begin
                  i = 0L
                  while(total(finite(qty[i,*])) eq 0) do i=i+1    ;in case the first time channel is masked
                  index1 = where(finite(qty[i,*]),count1)
                  if count1 ne n_elements(det_tt) then begin
                     qty = qty[*,index1]
                     det_tt = det_tt[index1]
                  endif
                  index1 = 0
               endif
               index = where((abs(det_tt) ge eqran[2]) and (abs(det_tt) le eqran[3]),count)
               if count eq 0 then begin
                  ok = 0b
                  ok = dialog_message('Two theta range is incompatible with the detector angles.',dialog_parent=self.tlb,/center)
                  break
               endif
               qty = qty[*,index] 
               if count gt 1 then qty = total(qty,2)/count
               ptr_free,self.bgtchnPtr
               self.bgtchnPtr = ptr_new({emean:emean,qty:qty})
               if n_elements(comment) eq 0 then comment=(*self.ftypename)[self.ftype]+' tchan. background data'
               self->set_filehistory,comment,open_path,open_file,ftpobj=ftpobj
               self->my_widget_control,'bgclBut',/sensitive
               if self.idl_version ge 5.6 then self->my_widget_control,'clearDat',tooltip='Background data will not be cleared.'
            endif
          end
        'bgdkbut':begin  ;shutter closed dark count file(s)
            self.filesel->getproperty,path=open_path,file=open_file,ftpobj=ftpobj
            if n_elements(open_file) eq 0 then begin
               ok = dialog_message(dialog_parent=self.tlb,'Please select dark count files first.',/error,/center)
               break
            endif
            case self.instrname of
              'dcs':begin     ;dcs data
                    self->dm_load_dcs_darkcount,open_path,open_file,parent=event.handler,error=error,comment=comment,bgrate=bgrate,dbgrate=dbgrate,ftpobj=ftpobj
                    end
              'spe':begin     ;spe file
                    ok = dialog_message('Not implemented.',dialog_parent=self.tlb,/center)
                    error = 1b
                    end
              'nxspe':begin   ;nxspe file
                    ok = dialog_message('Not implemented.',dialog_parent=self.tlb,/center)
                    error = 1b
                    end   
              'inx':begin     ;inx file
                    ok = dialog_message('Not implemented.',dialog_parent=self.tlb,/center)
                    error = 1b
                    end
              else: error = 1b ;do nothing
            endcase
            if keyword_set(error) then break
            if n_elements(bgrate) ne 0 then begin ;background file eg. empty can files
               ptr_free,self.bgratePtr
               self.bgratePtr = ptr_new([[bgrate],[dbgrate]])
               if n_elements(comment) eq 0 then comment=(*self.ftypename)[self.ftype]+' dark count file'
               self->set_filehistory,comment,open_path,open_file,ftpobj=ftpobj
               self->my_widget_control,'bgclBut',/sensitive
               if self.idl_version ge 5.6 then self->my_widget_control,'clearDat',tooltip='Background data will not be cleared.'
            endif
          end
        'vandbut':begin  ;load vanadium file(s)
            self.filesel->getproperty,path=open_path,file=open_file,ftpobj=ftpobj
            if n_elements(open_file) eq 0 then begin
               ok = dialog_message(dialog_parent=self.tlb,'Please select vanadium files first.',/error,/center)
               break
            endif
            case self.instrname of
              'dcs':begin     ;dcs data
                    self->dm_load_dcs_vanadium,open_path,open_file,parent=event.handler,error=error,comment=comment,qty=qty,ei=eief,info=info,ftpobj=ftpobj
                    end
              'macs':begin    ;macs file
                    self->dm_load_macs,open_path,open_file,a3=a3,deteff=deteff,ei=ei,ef=ef,error=error,info=info,cfx=cfx,comment=comment,$
                        kidney=kidney,parent=event.handler,qty=qty,dqty=dqty,temperature=temperature,weight=weight,ftpobj=ftpobj
                    end      
              'spe':begin     ;spe file
                    self->dm_load_spe,open_path,open_file,qty=qty,parent=event.handler,title='loading vanadium file...',error=error,eief=eief,phxfile=phxfile,info=info
                    end
              'nxspe':begin   ;nxspe file
                    self->dm_load_nxspe,open_path,open_file,qty=qty,parent=event.handler,title='loading vanadium file...',error=error,eief=eief,info=info
                    end   
              'inx':begin     ;inx file
                    self->dm_load_inx,open_path,open_file,qty=qty,parent=event.handler,title='loading vanadium file...',error=error,eief=eief,comment=comment,info=info
                    end
              'wand':begin    ;wand file
                    self->dm_load_wand,open_path,open_file,error=error,parent=event.handler,temperature=temperature,eief=eief,qty=qty,dqty=dqty,/sum
                    if ~keyword_set(error) then temperature = mean(temperature)
                    end
              else: error = 1b ;do nothing
            endcase
            if keyword_set(error) or (n_elements(qty) eq 0) then break
            if n_elements(temperature) eq 0 then temperature=300.0
            if n_elements(info)    eq 0 then info={temperature:mean(temperature)}
            if n_elements(comment) eq 0 then comment=(*self.ftypename)[self.ftype]+' vanadium data'
            if n_elements(phxfile) ne 0 then self.phxfile=phxfile
            self->set_filehistory,comment,open_path,open_file,ftpobj=ftpobj
            
            ;for MACS, first monitor normalization
            if self.instrname eq 'macs' then begin
               if self.macs_lamb2 then weight = dm_macs_lambda2correction(ei,weight,cfx)
               monfac = weight[0]/weight
               for i=0L,n_elements(monfac)-1 do qty[*,*,i] = qty[*,*,i]*monfac[i]
            endif
            
            ;check for empty can subtraction 
            if (self.samp_typ lt 2) and ptr_valid(self.bgdata) then begin  ;empty can subtraction
               if self.instrname eq 'macs' then subtract = 1b $
               else begin
                  size1 = size(qty,/dimensions)
                  size2 = size((*self.bgdata).qty,/dimensions)
                  subtract = (n_elements(size1) eq n_elements(size2) and (total(abs(size1-size2)) eq 0))
               endelse
               if subtract then begin
                  ans = dialog_message('Do you want to subtract the empty can data from the vanadium data?',/question,dialog_parent=self.tlb,/center)
                  if strupcase(ans) eq 'YES' then begin
                     if finite(self.ssfac) then ssfac=self.ssfac $
                     else ssfac = 1.0     ;self-shielding factor
                     if self.instrname eq 'macs' then begin
                        en = ei-ef & nen = n_elements(en)
                        bgqty = (*self.bgdata).qty
                        bgen  = (*self.bgdata).Ei-(*self.bgdata).Ef
                        bgkid = (*self.bgdata).kidney
                        nbgen = n_elements(bgen)
                        if self.macs_lamb2 then bgweight = dm_macs_lambda2correction((*self.bgdata).Ei,(*self.bgdata).weight,(*self.bgdata).cfx) $
                        else bgweight = (*self.bgdata).weight
                        monfac = weight[0]/bgweight
                        for i=0L,nbgen-1L do bgqty[*,*,i] = bgqty[*,*,i]*monfac[i]   
                        if ptr_valid(self.bgdata2) then begin
                           bgqty2 = (*self.bgdata2).qty
                           bgen2  = (*self.bgdata2).Ei-(*self.bgdata2).Ef
                           bgkid2 = (*self.bgdata2).kidney
                           nbgen2 = n_elements(bgen2)
                           if self.macs_lamb2 then bgweight = dm_macs_lambda2correction((*self.bgdata2).Ei,(*self.bgdata2).weight,(*self.bgdata2).cfx) $
                           else bgweight = (*self.bgdata2).weight
                           monfac = weight[0]/bgweight
                           for i=0L,nbgen2-1L do bgqty2[*,*,i] = bgqty2[*,*,i]*monfac[i]
                           ;apply the empty can factor
                           if total(finite(self.empcanfactor)) eq 2 then begin
                              bgqty  = bgqty*self.empcanfactor[0] 
                              bgqty2 = bgqty2*self.empcanfactor[1]
                           endif
                        endif
                        ;subtract bgqty
                        for i=0L,nen-1 do begin ;search order: En->kidney
                            if self.macs_bgefirst then begin    ;search order: En->kidney
                               tmp1 = min(abs(bgen-en[i]),imin) & tmp1 = bgen[imin]
                               if self.empt_tol_optn eq 0 then index1 = where((bgen eq tmp1) or (abs(bgen-en[i]) le self.macs_tolerance[0])) $
                               else index1 = where(abs(bgen-tmp1) le self.macs_tolerance[0])
                               tmp1 = min(abs(bgkid[index1]-kidney[i]),imin) & tmp1 = bgkid[index1[imin]]
                               if self.empt_tol_optn eq 0 then index2 = where((bgkid[index1] eq tmp1) or (abs(bgkid[index1]-kidney[i]) le self.macs_tolerance[1])) $
                               else index2 = where(abs(bgkid[index1]-tmp1) le self.macs_tolerance[1])
                            endif else begin                    ;search order: kidney->En
                               tmp1 = min(abs(bgkid-kidney[i]),imin) & tmp1 = bgkid[imin]
                               if self.empt_tol_optn eq 0 then index1 = where((bgkid eq tmp1) or (abs(bgkid-kidney[i]) le self.macs_tolerance[1])) $
                               else index1 = where(abs(bgkid-tmp1) le self.macs_tolerance[1])
                               tmp1 = min(abs(bgen[index1]-en[i]),imin) & tmp1 = bgen[index1[imin]]
                               if self.empt_tol_optn eq 0 then index2 = where((bgen[index1] eq tmp1) or (abs(bgen[index1]-en[i]) le self.macs_tolerance[0])) $
                               else index2 = where(abs(bgen[index1]-tmp1) le self.macs_tolerance[0])
                            endelse
                            index = index1[index2] 
                            if n_elements(index) eq 1 then qty[*,*,i] = qty[*,*,i]-bgqty[*,*,index[0]]*ssfac $
                            else qty[*,*,i] = qty[*,*,i]-total(bgqty[*,*,index],3)/n_elements(index)*ssfac
                        endfor
                        ;subtract bgqty2
                        if ptr_valid(self.bgdata2) then begin
                           for i=0L,nen-1 do begin ;search order: En->kidney
                               if self.macs_bgefirst then begin    ;search order: En->kidney
                                  tmp1 = min(abs(bgen2-en[i]),imin) & tmp1 = bgen2[imin]
                                  if self.empt_tol_optn eq 0 then index1 = where((bgen2 eq tmp1) or (abs(bgen2-en[i]) le self.macs_tolerance[0])) $
                                  else index1 = where(abs(bgen2-tmp1) le self.macs_tolerance[0])
                                  tmp1 = min(abs(bgkid2[index1]-kidney[i]),imin) & tmp1 = bgkid2[index1[imin]]
                                  if self.empt_tol_optn eq 0 then index2 = where((bgkid2[index1] eq tmp1) or (abs(bgkid2[index1]-kidney[i]) le self.macs_tolerance[1])) $
                                  else index2 = where(abs(bgkid2[index1]-tmp1) le self.macs_tolerance[1])
                               endif else begin                    ;search order: kidney->En
                                  tmp1 = min(abs(bgkid2-kidney[i]),imin) & tmp1 = bgkid2[imin]
                                  if self.empt_tol_optn eq 0 then index1 = where((bgkid2 eq tmp1) or (abs(bgkid2-kidney[i]) le self.macs_tolerance[1])) $
                                  else index1 = where(abs(bgkid2-tmp1) le self.macs_tolerance[1])
                                  tmp1 = min(abs(bgen2[index1]-en[i]),imin) & tmp1 = bgen2[index1[imin]]
                                  if self.empt_tol_optn eq 0 then index2 = where((bgen2[index1] eq tmp1) or (abs(bgen2[index1]-en[i]) le self.macs_tolerance[0])) $
                                  else index2 = where(abs(bgen2[index1]-tmp1) le self.macs_tolerance[0])
                               endelse
                               index = index1[index2]
                               if n_elements(index) eq 1 then qty[*,*,i] = qty[*,*,i]-bgqty2[*,*,index[0]]*ssfac $
                               else qty[*,*,i] = qty[*,*,i]-total(bgqty2[*,*,index],3)/n_elements(index)*ssfac
                           endfor
                        endif
                     endif else begin
                        if ptr_valid(self.bgdata2) then begin
                           if total(finite(self.empcanfactor)) eq 2 then begin
                              qty = qty-(*self.bgdata).qty*self.empcanfactor[0]*ssfac-(*self.bgdata2).qty*self.empcanfactor[1]*ssfac
                           endif else begin
                              qty = qty-(*self.bgdata).qty*ssfac-(*self.bgdata2).qty*ssfac
                           endelse
                        endif else begin
                           qty = qty-(*self.bgdata).qty*ssfac
                        endelse
                     endelse
                  endif
               endif
            endif
           
            if self.instrname eq 'macs' then begin
               ;ask for the a4 mask range
               effmask = dm_dialog_input(['ignore A4 from:','to:','calculation method:'],default=[self.macs_effmask,0],xsize=165,dialog_parent=self.tlb,$
                           droplist_content=ptr_new(['average of the intensity ratios','ratio of the averaged intensity']),$
                           is_droplist=[0,0,1],/return_number,info=['Data with A4 within the ignored range will not be used in the efficiency calculation.'],cancel=cancel)
               if keyword_set(cancel) then break
               if total(finite(effmask,/nan)) gt 0 then begin
                  ok = dialog_message('Invalid range. New detector efficiency is not calculated.',dialog_parent=self.tlb,/center)       
                  break
               endif   
               self.macs_effmask = effmask[0:1]
               eff = dm_calc_deteff(qty,kidney=kidney,maskdet=self.macs_effmask,intavg=round(effmask[2]))
               ind = where(finite(eff,/nan),count)
               if count ne 0 then begin
                  eff[ind] = deteff[ind]
                  for i=0,1 do eff[i,*] = eff[i,*]*20./total(eff[i,*])
               endif 
               ;now plot the data
               plottitle = ['original','new']+' det. efficiency'
               ploteffs  = reform([[[deteff[*,*]]],[[eff[*,*]]]])
               plotobjs  = objarr(2)
               if obj_valid(self.current) then begin
                  oldcurrent = self.current
                  self.current->getproperty,keep=oldkeep
                  self.current->setproperty,/keep
                  self.current = obj_new()
               endif
               det_tt = (*self.detPosPtr).two_theta
               for j=0,1 do begin
                   all = self.gr_list->get(/all,count=count)
                   for i=0,count-1 do begin
                       all[i]->getproperty,title=title
                       if title eq plottitle[j] then begin
                          self.current = all[i]
                          break
                       endif
                   endfor
                   if ~obj_valid(self.current) then begin
                      self.current = obj_new('dm_plot',parentobj=self,background=(['black','white'])[self.pbgcol_bw],/isolatin1,xsize=(*self.pdim_list)[0,self.pdim_choice],$
                           ysize=(*self.pdim_list)[1,self.pdim_choice],notooltip=self.pnotooltip)
                      self->dm_add_plotwindow,type=-1,title=plottitle[j]
                   endif else self.current->erase,/nodraw
                   plotobjs[j] = self.current    
                   xran = [0,0] & yran = [0,0]
                   colrs = self.current->getcolor(/list)
                   colrs = colrs[0:n_elements(colrs)-2] ;remove the user define color
                   for i=0,19 do begin
                       self.current->add_plot,kidney-76+det_tt[i],qty[0,i,*]*ploteffs[0,i,j],color=colrs[i mod n_elements(colrs)],psym='circle',linestyle='no line',legend=dm_to_string(i+1)
                       self.current->add_plot,kidney-76+det_tt[i],qty[1,i,*]*ploteffs[1,i,j],color=colrs[i mod n_elements(colrs)],psym='circle',linestyle='no line'
                       xmin = min(kidney-76+det_tt[i],max=xmax)
                       xran = [(xran[0])<(xmin),(xran[1])>(xmax)]
                       ymin = min(qty[0,i,*]*ploteffs[0,i,j],max=ymax)
                       yran = [(yran[0])<(ymin),(yran[1])>(ymax)]
                       ymin = min(qty[1,i,*]*ploteffs[1,i,j],max=ymax)
                       yran = [(yran[0])<(ymin),(yran[1])>(ymax)]
                   endfor
                   self.current->setproperty,xtit='A4 (\deg)',ytit='Intensity (counts/monitor='+dm_to_string(weight[0],/exp)+')',xran=xran+[-5,5],yran=yran+[-0.1,0.1]*(yran[1]-yran[0]),$
                       title=plottitle[j],/showygrid,/keep,/hidelegend
               endfor
               if n_elements(oldcurrent) ne 0 then begin
                  self.current = oldcurrent
                  self.current->setproperty,keep=oldkeep
               endif
               ptr_free,self.eff1
               self.eff1 = ptr_new(deteff,/no_copy)
               self->dm_arrange_plot,plotobjs
               info = '    SPEC       DIFF'
               for i=0,19 do info = [info,dm_to_string(eff[*,i],strlength=12,sep=',')]
               ok = dialog_message(info,dialog_parent=self.tlb,/center,/info,title='det efficiency factor')
            endif else begin
               ;for SPE and NXSPE and WAND file type, check masks
               if self.ftypecheckmask then begin
                  i = 0L
                  while(total(finite(qty[i,*])) eq 0) do i=i+1    ;in case the first time channel is masked
                  maskdet = where(finite(qty[i,*],/nan),count)
                  if count eq 0 then tmp=temporary(maskdet)
               endif
               
               ;Debye Waller factor correction
               if n_tags(info) ne 0 then begin
                  tnames = tag_names(info)
                  index  = where(tnames eq 'TEMPERATURE',count)
                  if count ne 0 then $
                     temperature = info.temperature $
                  else $
                     temperature = 300.0
               endif
               temperature = dm_dialog_input('Vanadium Temperature(K):',default=dm_to_string(temperature),title='Debye-Waller Factor',$
                  info=['For the Debye-Waller factor of vanadium.','Press Skip to skip the Debye-Waller factor calculation.'],dialog_parent=self.tlb,cancel=cancel,cbstring='Skip')
               dw = 1.0
               if ~keyword_set(cancel) then begin 
                  if strlen(temperature) ne 0 then begin
                     temperature = dm_to_number(temperature,/float)
                     if (~ finite(temperature)) then begin
                        ok  = dialog_message('Invalid inputs. Debye-Waller factor for vanadium is not calculated.',dialog_parent=self.tlb,/center)       
                     endif else begin
                        tth = (*self.detPosPtr).two_theta
                        dw  = dm_calc_dwfactor(temperature,tth,eief,maskdet=maskdet)
                     endelse
                  endif
               endif
               if size(qty,/n_dimensions) eq 1 then qty = reform(qty,1,n_elements(qty))
               eff = dm_calc_deteff(qty,/fit,maskdet=maskdet)/dw & maskdet=0
               
               ;add absorption correction
               mesg = [ 'Press Skip to skip the absorption correction for vanadium file,',$
                        'or press OK without filling in all the information.',$
                        '',$
                        'L0 is the energy independent mean free path;',$
                        'L1 is the energy dependent mean free path for',$
                        '2200m/s or 25.3 meV neutron.',$
                        'The effective mean free path is:',$
                        'L=1/{1/L0+1/(L1*sqrt[E(meV)/25.3])}' ]
               default = ['2.8','2.7','','']
               parm = dm_dialog_input(['L0(cm):','L1(cm):','Rout(cm):','Rin(cm):'],default=default,info=mesg,title='Absorption correction for vanadium',$
                      xsize=140,dialog_parent=self.tlb,cancel=cancel,cbstring='Skip')
               if ~keyword_set(cancel) then begin     
                  if min(strlen(parm)) gt 0 then begin
                     parm = dm_to_number(parm,/float)
                     if total(finite(parm)) eq 4 then begin
                        tth    = (*self.detPosPtr).two_theta
                        factor = dm_calc_shielding([1,parm],eief,0,tth,group_leader=self.tlb)
                        eff    = eff*factor
                     endif else begin
                        ok = dialog_message('Invalid inputs. Absorption correction for vanadium sample is disabled.',dialog_parent=self.tlb,/center)
                     endelse
                  endif
               endif
            endelse
            
            ptr_free,self.eff
            self.eff = ptr_new(eff,/no_copy)
            self->my_widget_control,'vdclBut',/sensitive
            if self.instrname eq 'macs' then self->my_widget_control,'macsdeteffBut',/sensitive
          end
        'loadbut':begin  ;load data file(s)
            if n_elements(event) ne 0 then $
               if event.select eq 0 then break
            self.filesel->getproperty,path=open_path,file=open_file,ftpobj=ftpobj
            if ~obj_valid(ftpobj) then self->set_dir,dataDir=open_path
            if keyword_set(scriptmode) and n_elements(filename) ne 0  then open_file=filename
            if keyword_set(scriptmode) and n_elements(directory) ne 0 then open_path=directory
            if n_elements(open_file) eq 0 then begin
               ok = dialog_message(dialog_parent=self.tlb,'Please select data files first.',/error,/center)
               break
            endif
            
            ;clear the data and projection pointer
            ptr_free,self.kfactor,self.dataStrPtr,self.projStrPtr
            self->my_widget_control,['addBut','clearDat','mkplBut'],sensitive=0  
            self->set_display_button
            
            case self.instrname of
              'dcs':begin     ;dcs data
                    self->dm_load_dcs,open_path,open_file,parent=event.handler,error=error,$
                        comment=comment,qty=qty,dqty=dqty,weight=weight,ang=ang,ei=eief,info=info,$
                        ewid=ewid,emean=emean,monrate=monrate,kfactor=kfactor,tmean=tmean,adddata=ptr_valid(self.bgdata),ftpobj=ftpobj
                    end
              'macs':begin
                    self->dm_load_macs,open_path,open_file,a3=a3,deteff=deteff,ei=ei,ef=ef,error=error,info=info,cfx=cfx,comment=comment,$
                        kidney=kidney,parent=event.handler,qty=qty,dqty=dqty,temperature=temperature,weight=weight,ftpobj=ftpobj
                    end      
              'spe':begin     ;spe file
                    e_range = self.e_range
                    self->dm_load_spe,open_path,open_file,parent=event.handler,diffuse=(self.samp_typ ge 2),$
                        phxfile=phxfile,avgsum=self.bin_avgsum,e_range=e_range,t_chan=self.e_bin[self.samp_typ],error=error,$
                        qty=qty,dqty=dqty,eief=eief,ang=ang,emean=emean,info=info,ewid=ewid,weight=weight,$
                        eint_yn=self.eint_yn,kfactor=kfactor,diffetyp=(self.samp_typ eq 3),adddata=ptr_valid(self.bgdata),$
                        scriptmode=scriptmode
                    if (self.samp_typ ge 2) and (n_elements(e_range) ne 0) then self.e_range=e_range
                    end
              'nxspe':begin   ;nxspe file
                    e_range = self.e_range
                    self->dm_load_nxspe,open_path,open_file,parent=event.handler,diffuse=(self.samp_typ ge 2),$
                        avgsum=self.bin_avgsum,e_range=e_range,t_chan=self.e_bin[self.samp_typ],error=error,$
                        qty=qty,dqty=dqty,eief=eief,ang=ang,emean=emean,info=info,lamcorr=lamcorr,ewid=ewid,weight=weight,$
                        eint_yn=self.eint_yn,kfactor=kfactor,diffetyp=(self.samp_typ eq 3),adddata=ptr_valid(self.bgdata),$
                        scriptmode=scriptmode
                    if (self.samp_typ ge 2) and (n_elements(e_range) ne 0) then self.e_range=e_range
                    if ~keyword_set(error) then self.lamcorrnxspe = lamcorr
                    end   
              'inx':begin     ;inx file
                    e_range = self.e_range
                    self->dm_load_inx,open_path,open_file,parent=event.handler,diffuse=(self.samp_typ ge 2),$
                        avgsum=self.bin_avgsum,e_range=e_range,t_chan=self.e_bin[self.samp_typ],error=error,$
                        qty=qty,dqty=dqty,eief=eief,ang=ang,emean=emean,info=info,ewid=ewid,eint_yn=self.eint_yn,weight=weight,$
                        kfactor=kfactor,diffetyp=(self.samp_typ eq 3),comment=comment,adddata=ptr_valid(self.bgdata),$
                        scriptmode=scriptmode
                    if (self.samp_typ ge 2) and (n_elements(e_range) ne 0) then self.e_range=e_range
                    end
              'wand':begin    ;wand file
                    self->dm_load_wand,open_path,open_file,ang=ang,error=error,parent=event.handler,qty=qty,dqty=dqty,eief=eief,emean=emean,$
                        temperature=temperature,weight=weight,scriptmode=scriptmode
                    if ~keyword_set(error) and ~self.extravaxis_yn then legend = 'T='+dm_to_string(mean(temperature),resolution=2)+' K'
                    end
              else: error = 1b ;do nothing
            endcase
            if keyword_set(error) or (n_elements(qty) eq 0) then break
            if n_elements(temperature) eq 0 then temperature = 300.0
            t_mean = mean(temperature)
            if n_elements(monrate) eq 0 then monrate = 1.0
            if n_elements(kfactor) eq 0 then kfactor = 1.0
            if n_elements(info)    eq 0 then info = {temperature:t_mean}
            if n_elements(tmean)   eq 0 then tmean = 0.0
            if n_elements(phxfile) ne 0 then self.phxfile = phxfile
            self.monrate = monrate
            self.binthreshold = 0.0 ;reset the binning threshold
            self.userfwarning = 1b  ;reset the warning flag

            if self.instrname eq 'macs' then begin
               self.psi[3] = A3[0] ;save the first A3 angle
               dm_combine_macs,qty,dqty,weight,cfx,A3,kidney,Ei,Ef,temperature=temperature,eresolution=3,viewt=self.extravaxis_yn
               self.macs_mon[0] = min(weight)
               self->my_widget_control,'monSum',set_value='Automatic ('+dm_to_string(self.macs_mon[0],/exponent)+')'
               self->reset_intnlab
               kfactor = sqrt(Ei/Ef)
               ;set projection parameters
               if ~self.locklattuv then begin
                  unames = ['latt'+['A','B','C','AA','BB','CC'],'orie'+['U'+['x','y','z'],'V'+['x','y','z']]]
                  values = [info.latticeparm,info.latticeori]
                  self->script,'set_parameter',name=unames,value=values
                  ;check viewing axes to see if they are on the scattering plane, if not, replace them
                  if ~(self->viewonscattplane()) then self->script,'set_parameter',name='viewU'+['1'+['h','k','l'],'2'+['h','k','l']],value=info.latticeori
               endif
               ;check instrument geometry
               minEi = min(Ei,max=maxEi) & minEf = min(Ef,max=maxEf)
               if (self.instrgeom eq 1) and (abs(minEf-maxEf) gt 0.002) then switchgeom = 1b  ;inverse geometry
               if (self.instrgeom eq 0) and (abs(minEi-maxEi) gt 0.002) then switchgeom = 1b  ;direct geometry  
               if keyword_set(switchgeom) then begin
                  ans = dialog_message(dialog_parent=self.tlb,['The instrument might be in the '+(['inverse','direct'])[self.instrgeom]+' geometry mode.',$
                         'Do you want to switch to the '+(['inverse','direct'])[self.instrgeom]+' geometry?'],/question,/center)
                  if strupcase(ans) eq 'YES' then self->set_instrgeom,geom=(1-self.instrgeom)
               endif
               self.eief = ([(minEi+maxEi)/2.0,(minEf+maxEf)/2.0])[self.instrgeom]
               self.estartend = dm_startend(Ei-Ef,resolution=3)
               self.kidney    = dm_startend(kidney,tolerance=0.1)
               self.psi[0:2]  = dm_startend(A3,resolution=2)
               self.dataStrPtr = ptr_new({qty:temporary(qty),err:temporary(dqty),rotation:temporary(A3),kidney:temporary(kidney),weight:temporary(weight),cfx:temporary(cfx),Ei:temporary(Ei),$
                    Ef:temporary(Ef),temperature:temporary(temperature),info:temporary(info)})
               if n_elements(deteff) eq 2* n_elements((*self.detPosPtr).two_theta) then begin
                  if ~ptr_valid(self.eff1) then begin
                     if self.ftoolflag[1] then begin
                        ans = dialog_message(dialog_parent=self.tlb,'Do you want to overwrite the detector efficiency by the efficiency info stored in the data file?',/question,/center,/DEFAULT_NO)
                        if strupcase(ans) eq 'YES' then self.ftoolflag[1] = 0b
                     endif
                     if ~self.ftoolflag[1] then begin
                        ptr_free,self.eff
                        self.eff = ptr_new(deteff)   
                     endif
                  endif else begin  ;update the backup efficiency pointer
                     ptr_free,self.eff1
                     self.eff1 = ptr_new(deteff)   
                  endelse
               endif
               self->my_widget_control,'macsdeteffBut',sensitive=ptr_valid(self.eff)
               if ptr_valid(self.mask) and (self.masktype eq 1) then self->clear_mask,/message
            endif else begin
               self.eief = abs(eief)
               ;readjust psi value inteligently
               if (n_elements(ang) ne 0) and finite(self.orie_offset) and (self.samp_typ ge 1) then begin
                  oldpsi = self.orie_psi
                  self->script,'set_parameter',name='oriePsi',value=ang[0]+self.orie_offset,resolution=3
                  ;readjust flat plate absorption angle intelligently
                  if ptr_valid(self.absp_info) and finite(oldpsi) and (~keyword_set(scriptmode)) then begin
                     if ((*(self.absp_info))[0] eq 3) and ((self.orie_psi-oldpsi) ne 0) then begin  ;flat plate
                        (*(self.absp_info))[n_elements(*(self.absp_info))-1] = (*(self.absp_info))[n_elements(*(self.absp_info))-1]+self.orie_psi-oldpsi
                        ok = dialog_message(dialog_parent=self.tlb,'Please check the absorption correction flat plate angle.',/info,/center)
                     endif
                  endif
               endif
               ;check instrument geometry to see if it is consistent with the energy
               if ((self.instrgeom eq 0) and (max(emean) ge eief)) or ((self.instrgeom eq 1) and (min(emean) le -eief)) then begin
                  ans = dialog_message(dialog_parent=self.tlb,['The instrument might be '+(['an inverse','a direct'])[self.instrgeom]+' geometry one.',$
                         'Do you want to switch to the '+(['inverse','direct'])[self.instrgeom]+' geometry?'],/question,/center)
                  if strupcase(ans) eq 'YES' then self->set_instrgeom,geom=(1-self.instrgeom)
               endif
            
               if self.samp_typ ge 2 then begin
                  nang = n_elements(ang)
                  if n_elements(weight) eq 0 then weight=fltarr(nang)+1.0
                  if (self.samp_typ eq 3) and (self.e_bin[4] gt 1) then begin
                     dm_grid_ebin,self.e_bin[4],qty,dqty,emean,avgsum=self.bin_avgsum
                     if n_elements(kfactor) gt 1 then $
                        dm_grid_ebin,self.e_bin[4],kfactor $
                     else begin
                        if self.instrgeom eq 0 then $
                           kfactor = sqrt(eief/(eief-emean)) $
                        else $
                           kfactor = sqrt((eief+emean)/eief)
                     endelse
                     if n_elements(ewid) ne 0 then dm_grid_ebin,self.e_bin[4],ewid
                  endif
                  if n_elements(ewid) eq 0 then ewid=1.0
                  self.psi[3]   = ang[0]  ;save the angle of first file
                  self.psi[0:2] = dm_startend(ang,resolution=2)
                  ;combine data with the same psi angles
                  if (self.instrname eq 'wand') and self.extravaxis_yn then begin ;temperature is a viewing axis
                     ndat = n_elements(weight)
                     tmp = dm_to_string(ang,resolution=2)+'_'+dm_to_string(temperature,resolution=2)
                     dm_combine_psi,tmp,qty,dqty,weight,checkfinite=self.ftypecheckmask
                     ndat1 = n_elements(weight) 
                     if ndat1 ne ndat then begin
                        ang = fltarr(ndat1) & temperature = fltarr(ndat1)
                        for i=0L,ndat1-1 do begin
                            tmp1 = dm_to_number(strsplit(tmp[i],'_',/extract,count=count))
                            if count eq 2 then begin
                               ang[i] = tmp1[0]
                               temperature[i] = tmp1[1]
                            endif
                        endfor
                     endif
                  endif else $
                     dm_combine_psi,ang,qty,dqty,weight,checkfinite=self.ftypecheckmask,extra=temperature
               endif else begin
                  if n_elements(weight) eq 0 then weight=1.0
                  if n_elements(ewid) eq 0   then ewid=fltarr(n_elements(emean))+1.0
                  if n_elements(ang) eq 0    then ang=!values.f_nan
               endelse
               self.dataStrPtr = ptr_new({qty:temporary(qty),err:temporary(dqty),weight:temporary(weight),temperature:temporary(temperature),$
                    rotation:temporary(ang),info:temporary(info),eief:temporary(eief),ewid:temporary(ewid),e:temporary(emean),$
                    time:tmean,filename:open_file,filepath:open_path},/no_copy)
               if finite(self.orie_psi,/nan) and (self.samp_typ ge 2) then begin
                  self->script,'set_parameter',name='oriePsi',value=(*self.dataStrPtr).rotation[0],resolution=3
               endif
            endelse   
            self.kfactor = ptr_new(kfactor,/no_copy)
            self->my_widget_control,['addBut','clearDat','mkplBut'],/sensitive    ;allow adding data & clear data & plot mask
            self->set_display_button
            if n_elements(legend) eq 0 then legend = ''
            if n_elements(comment) eq 0 then begin
               comment = '' 
            endif else begin    ;legend and comments for dm_plot
               if stregex(comment,'T ?= *$',/boolean) then begin    ;patch the comment ending with "T=" with the temerature set point
                  tmp = t_mean
                  field = tag_names(*self.dataStrPtr)
                  loc  = where(strupcase(field) eq 'INFO',count)
                  if count gt 0 then begin
                     field = tag_names((*self.dataStrPtr).info)
                     loc  = where(strupcase(field) eq 'TEMPERATURE',count)
                     if count gt 0 then tmp = (*self.dataStrPtr).info.temperature
                  endif
                  comment = comment+dm_to_string(tmp[0])+' K'  
               endif
               tmpcomment = ' '+comment+' '
               units = '(mk)?(k)?(tesla)?(t)?(gauss)?(kpa)?(mpa)?(gpa)?(pa)?(mtorr)?(torr)?(mbar)?(kbar)?(bar)?(psi)?(ksi)?(atm)?(mev)?(ev)?' ;unit string
               repeat begin 
                  tmp = stregex(tmpcomment,' +[tbhpe]? *[=~<>]? *[0-9p.-]+ *'+units+'[ ,;]+',length=len,/subexpr,/fold_case)
                  ind = where(len gt 0,count)
                  if count le 1 then begin
                     tmp = stregex(tmpcomment,'[=~<>]?[0-9p.-]+ *'+units+'[ ,;]+',length=len,/subexpr,/fold_case)
                     ind = where(len gt 0,count) 
                     if count eq 1 then tmpcomment = ' '+strmid(tmpcomment,0,tmp[0])+' '+strmid(tmpcomment,tmp[0]+len[0])+' '
                  endif
                  if count gt 1 then begin
                     for i=1,count-1 do begin
                         tmp1 = strtrim(strmid(tmpcomment,0,tmp[ind[i]]),2)
                         if strlowcase(strmid(tmp1,0,1,/reverse)) ne 'p' then begin ;if p is part of a number, like 1p8, it can't be alone
                            legend = [legend,strtrim(strmid(tmpcomment,tmp[0],len[0]),2)]
                            break
                         endif
                     endfor
                     tmpcomment = ' '+strmid(tmpcomment,0,tmp[0])+' '+strmid(tmpcomment,tmp[0]+len[0])+' '
                  endif
               endrep until count lt 1
               if n_elements(legend) gt 1 then begin  ;compile legend string according to their positions
                  ind = where(strlen(legend) gt 0,count)
                  if count gt 0 then tmplegend = legend[ind] else tmplegend = legend[0]
                  count = n_elements(tmplegend)
                  pos = lonarr(count)
                  for i=0,count-1 do pos[i] = strpos(comment,tmplegend[i])
                  ind = sort(pos) & legend = ''
                  for i=0,count-1 do legend = legend+((strlen(legend) gt 0)?', ':'')+tmplegend[ind[i]]
                  legend = strtrim(legend,2)
               endif
               if strlen(legend) eq 0 then begin      ;use the last word as the legend if no legend found based on preset criterion
                  tmp    = strsplit(comment,', ;'+string(9b),/extract) & ntmp = n_elements(tmp)
                  legend = tmp[(0>(ntmp-1))]
               endif
            endelse
            self->script,'set_parameter',name='legdLab',value=legend
            self->script,'set_parameter',name='titlLab',value=comment
            if strlen(comment) eq 0 then comment=(*self.ftypename)[self.ftype]+' data '+open_file[0]+(n_elements(open_file) gt 1?'...':'')
            self->set_filehistory,comment,open_path,open_file,ftpobj=ftpobj
          end
        'addbut':begin   ;add dcs data file to current data pointer
            if n_elements(event) ne 0 then $
               if event.select eq 0 then break
            self.filesel->getproperty,path=open_path,file=open_file,ftpobj=ftpobj
            if n_elements(open_file) eq 0 then begin
               ok = dialog_message(dialog_parent=self.tlb,'Please select data files first.',/error,/center)
               break
            endif
            case self.instrname of
              'dcs':begin     ;dcs data
                    info = (*self.dataStrPtr).info
                    self->dm_load_dcs,open_path,open_file,parent=event.handler,error=error,comment=comment,qty=qty,dqty=dqty,weight=weight,ang=ang,$
                        info=info,title='adding data file...',/adddata,ftpobj=ftpobj
                    end
              'macs':begin    ;macs data
                    info = (*self.dataStrPtr).info
                    self->dm_load_macs,open_path,open_file,a3=a3,ei=ei,ef=ef,error=error,info=info,cfx=cfx,comment=comment,$
                        kidney=kidney,parent=event.handler,qty=qty,dqty=dqty,temperature=temperature,weight=weight,ftpobj=ftpobj
                    end
              'spe':begin     ;spe file
                    info = (*self.dataStrPtr).info
                    e_range = self.e_range
                    self->dm_load_spe,open_path,open_file,parent=event.handler,diffuse=(self.samp_typ ge 2),weight=weight,$
                        avgsum=self.bin_avgsum,e_range=e_range,t_chan=self.e_bin[self.samp_typ],error=error,qty=qty,dqty=dqty,$
                        eief=eief,ang=ang,emean=emean,info=info,eint_yn=self.eint_yn,diffetyp=(self.samp_typ eq 3),/adddata
                    end
              'nxspe':begin   ;nxspe file
                    info = (*self.dataStrPtr).info
                    e_range = self.e_range
                    self->dm_load_nxspe,open_path,open_file,parent=event.handler,diffuse=(self.samp_typ ge 2),weight=weight,$
                        avgsum=self.bin_avgsum,e_range=e_range,t_chan=self.e_bin[self.samp_typ],error=error,qty=qty,dqty=dqty,$
                        eief=eief,ang=ang,emean=emean,info=info,eint_yn=self.eint_yn,diffetyp=(self.samp_typ eq 3),/adddata
                    end
              'inx':begin     ;inx file
                    info = (*self.dataStrPtr).info
                    e_range = self.e_range
                    self->dm_load_inx,open_path,open_file,parent=event.handler,diffuse=(self.samp_typ ge 2),weight=weight,$
                        avgsum=self.bin_avgsum,e_range=e_range,t_chan=self.e_bin[self.samp_typ],error=error,qty=qty,dqty=dqty,$
                        eief=eief,ang=ang,emean=emean,info=info,eint_yn=self.eint_yn,diffetyp=(self.samp_typ eq 3),/adddata
                    end
              'wand':begin    ;wand file
                    self->dm_load_wand,open_path,open_file,ang=ang,error=error,parent=event.handler,qty=qty,dqty=dqty,temperature=temperature,$
                        eief=eief,emean=emean,weight=weight
                    end                    
              else: error = 1b ;do nothing
            endcase
            if keyword_set(error) or (n_elements(qty) eq 0) then break
            if n_elements(comment) eq 0 then comment=(*self.ftypename)[self.ftype]+' data'+open_file[0]+(n_elements(open_file) gt 1?'...':'')
            self->set_filehistory,comment,open_path,open_file,ftpobj=ftpobj
            if self.instrname eq 'macs' then begin
               info   = (*self.dataStrPtr).info
               qty    = [[[(*self.dataStrPtr).qty]],[[temporary(qty)]]]
               err    = [[[(*self.dataStrPtr).err]],[[temporary(dqty)]]]
               weight = [(*self.dataStrPtr).weight,temporary(weight)]
               cfx    = [(*self.dataStrPtr).cfx,temporary(cfx)]
               A3     = [(*self.dataStrPtr).rotation,temporary(A3)]
               kidney = [(*self.dataStrPtr).kidney,temporary(kidney)]
               Ei     = [(*self.dataStrPtr).Ei,temporary(Ei)]
               Ef     = [(*self.dataStrPtr).Ef,temporary(Ef)]
               temperature = [(*self.dataStrPtr).temperature,temporary(temperature)]
               ;combine data
               dm_combine_macs,qty,err,weight,cfx,A3,kidney,Ei,Ef,temperature=temperature,eresolution=3,viewt=self.extravaxis_yn
               ptr_free,self.kfactor,self.dataStrPtr
               self.kfactor   = ptr_new(sqrt(Ei/Ef))
               self.estartend = dm_startend(Ei-Ef,resolution=3)
               self.kidney    = dm_startend(kidney,tolerance=0.1)
               self.psi[0:2]  = dm_startend(A3,resolution=2)
               self.dataStrPtr = ptr_new({qty:temporary(qty),err:temporary(err),rotation:temporary(A3),kidney:temporary(kidney),weight:temporary(weight),cfx:temporary(cfx),Ei:temporary(Ei),$
                                    Ef:temporary(Ef),temperature:temporary(temperature),info:temporary(info)})
               if ptr_valid(self.mask) and (self.masktype eq 1) then self->clear_mask,/message                     
            endif else begin
               if self.samp_typ ge 2 then begin
                  if n_elements(weight) eq 0 then weight = fltarr(n_elements(ang))+1.0
                  if self.samp_typ eq 3 then begin
                     if self.e_bin[4] gt 1 then begin
                        tmp = size(qty,/dimension)
                        dm_grid_ebin,self.e_bin[4],qty,dqty,fltarr(tmp[0]),avgsum=self.bin_avgsum
                     endif
                     qty = [[[(*self.dataStrPtr).qty]],[[temporary(qty)]]]
                     err = [[[(*self.dataStrPtr).err]],[[temporary(dqty)]]]
                  endif else begin
                     qty = [[(*self.dataStrPtr).qty],[temporary(qty)]]
                     err = [[(*self.dataStrPtr).err],[temporary(dqty)]]
                  endelse
                  weight = [(*self.dataStrPtr).weight,temporary(weight)]
                  ang    = [(*self.dataStrPtr).rotation,temporary(ang)]
                  filename = [(*self.dataStrPtr).filename,open_file]
                  e      = (*self.dataStrPtr).e
                  eief   = (*self.dataStrPtr).eief
                  info   = (*self.dataStrPtr).info
                  ewid   = (*self.dataStrPtr).ewid
                  path   = (*self.dataStrPtr).filepath
                  time   = (*self.dataStrPtr).time
                  
                  if self.instrname eq 'wand' then temperature = [(*self.dataStrPtr).temperature,temperature] $
                  else temperature = (*self.dataStrPtr).temperature
                  ;combine data with the same psi angles
                  if (self.instrname eq 'wand') and self.extravaxis_yn then begin ;temperature is a viewing axis
                     ndat = n_elements(weight)
                     tmp = dm_to_string(ang,resolution=2)+'_'+dm_to_string(temperature,resolution=2)
                     dm_combine_psi,tmp,qty,err,weight,checkfinite=self.ftypecheckmask
                     ndat1 = n_elements(weight) 
                     if ndat1 ne ndat then begin
                        ang = fltarr(ndat1) & temperature = fltarr(ndat1)
                        for i=0L,ndat1-1 do begin
                            tmp1 = dm_to_number(strsplit(tmp[i],'_',/extract,count=count))
                            if count eq 2 then begin
                               ang[i] = tmp1[0]
                               temperature[i] = tmp1[1]
                            endif
                        endfor
                     endif
                  endif else $
                     dm_combine_psi,ang,qty,err,weight,checkfinite=self.ftypecheckmask,extra=temperature
                  self.psi[0:2] = dm_startend(ang,resolution=2)
                  ptr_free,self.dataStrPtr
                  self.dataStrPtr = ptr_new({qty:temporary(qty),err:temporary(err),weight:temporary(weight),temperature:temporary(temperature),$
                      rotation:temporary(ang),info:temporary(info),eief:temporary(eief),ewid:temporary(ewid),$
                      e:temporary(e),time:time,filename:temporary(filename),filepath:path},/no_copy)
               endif else begin
                  if n_elements(weight) eq 0 then weight = 1.0
                  (*self.dataStrPtr).qty = ((*self.dataStrPtr).qty)*((*self.dataStrPtr).weight)+qty*weight
                  (*self.dataStrPtr).err = sqrt((((*self.dataStrPtr).err)*((*self.dataStrPtr).weight))^2+(dqty*weight)^2)
                  (*self.dataStrPtr).weight = ((*self.dataStrPtr).weight)+weight
                  (*self.dataStrPtr).qty = ((*self.dataStrPtr).qty)/((*self.dataStrPtr).weight)
                  (*self.dataStrPtr).err = ((*self.dataStrPtr).err)/((*self.dataStrPtr).weight)
               endelse
            endelse   
          end
        'ftype_':   begin
            if self.ftype eq type then break   ;no change
            case self.instrname of    ;giving warning of losing the detector offset value
                 'macs':  warning = (*self.detPosPtr).two_theta[0] ne 0
                 'wand':  warning = (*self.detPosPtr).two_theta[0] ne -11.236
                 else:
            endcase
            if keyword_set(warning) then ok = dialog_message(dialog_parent=self.tlb,'The detector offset angle info will be lost.',/info,/center)
            self.ftype = type
            ;remove all relevant data
            if (~ keyword_set(init)) then begin
               ptr_free,self.detPosPtr,self.dataStrPtr,self.projStrPtr
               self->set_display_button
               ;clear all possible background data and mask
               self.monrate = 1.0
               ptr_free,self.eff,self.eff1,self.bgtchnPtr,self.bgratePtr,self.bgdetsPtr,self.bgdata,self.bgdata2,self.mask,self.a3mask
               self.dcsdetmask[*] = 0
            endif
            self->dm_filemenu       ;reset file menu and other menus
            ;change the about ... information in the help menu
            (*self.info)[0] = '--Mslice for '+(*self.ftypename)[type]+' --'
            widget_control,self.tlb,tlb_set_title=(*self.info)[0]
            self->my_widget_control,'selfBut',set_value='About Mslice for '+(*self.ftypename)[type]
            if obj_valid(self.fileSel) then self.fileSel->set_filter,'.'+(*self.ftypeext)[type]          ;change file filter
          end

        ;DCS tools
        'dcs_cspebut':   dm_dcs2spe,event,workdir=self.dirs[1],datadir=self.dirs[0]  ;convert dcs files to spe files
        'dcs_algnbut':   dcs_cryst_align,event,[self.latt_parm,self.orie_u,self.orie_v],workdir=self.dirs[1],datadir=self.dirs[0],bgcolor=(['black','white'])[self.pbgcol_bw] 
        'dcs_convbut':   begin
                         dm_ereslconv,event,workdir=self.dirs[1],bgcolor=(['black','white'])[self.pbgcol_bw],noinfo=self.ftoolflag[0],notooltip=self.pnotooltip,resltype=2 
                         self.ftoolflag[0] = 1b  ;no dialog message next time
                         end
        'dcs_patchbut':  dm_dcspatch,event,dataDir=self.dirs[0],workDir=self.dirs[1]
        'dm_copyftpbut': dm_copyftp,event,workDir=self.dirs[1],filter='.'+(*self.ftypeext)[self.ftype]   ;also available as macs file tool
        ;MACS tools
        'macs_algnbut':  dcs_cryst_align,event,[self.latt_parm,self.orie_u,self.orie_v],workdir=self.dirs[1],datadir=self.dirs[0],bgcolor=(['black','white'])[self.pbgcol_bw],/macs
        ;WAND tools
        'wand_algnbut':  dcs_cryst_align,event,[self.latt_parm,self.orie_u,self.orie_v],workdir=self.dirs[1],datadir=self.dirs[0],bgcolor=(['black','white'])[self.pbgcol_bw],/wand
        ;put the events of your own instrument specific tools below

        else: handled = 0b      ;event not handled
    endcase

    if obj_valid(ftpboj) then ftpobj->CloseConnections

    return,handled
end