; $Id: dm_to_number.pro,v 1.10.2.1 2013/05/06 17:56:04 ymqiu Exp $
;#######################################################################
;
; NAME:
;  dm_to_number
;
; PURPOSE:
;  convert string to numbers,default is to convert to a float
;  if the string is empty, return NaN for float or double type
;  return 0 for int,long or byte type
;  return 0 if the string matched any zerostring
;  return 1 if the string matched any onestring
;  return 2 if the string matcched any twostring
;  
; CATEGORY:
;  general
;
; AUTHOR:
;  Yiming Qiu
;  NIST Center for Neutron Research
;  100 Bureau Drive, Gaithersburg, MD 20899-6102
;  United States
;  yiming.qiu@nist.gov
;  April, 2013
;
; LICENSE:
;  The software in this file is written by an employee of
;  National Institute of Standards and Technology
;  as part of the DAVE software project.
;
;  The DAVE software package is not subject to copyright protection
;  and is in the public domain. It should be considered as an
;  experimental neutron scattering data reduction, visualization, and
;  analysis system. As such, the authors assume no responsibility
;  whatsoever for its use, and make no guarantees, expressed or
;  implied, about its quality, reliability, or any other
;  characteristic. The use of certain trade names or commercial
;  products does not imply any endorsement of a particular product,
;  nor does it imply that the named product is necessarily the best
;  product for the stated purpose. We would appreciate acknowledgment
;  if the DAVE software is used or if the code in this file is
;  included in another product.
;
;#######################################################################

function dm_to_number,numstr,float=float,int=int,double=double,byte=byte,long=long,count=count,date=date,zerostring=zerostring,onestring=onestring,twostring=twostring
    count = n_elements(numstr)
    if count eq 0 then return, 0
    tmp = size(numstr,/dimension) 
    if count eq 1 then tmp = 1
    str = numstr 
    if size(numstr[0],/type) ne 7 then str = strtrim(string(numstr),2)
    if keyword_set(int)           then val = intarr(tmp)    $
    else if keyword_set(long)     then val = lonarr(tmp)    $
    else if keyword_set(double)   then val = dblarr(tmp)    $
    else if keyword_set(byte)     then val = bytarr(tmp)    $
    else if keyword_set(date)     then val = ulon64arr(tmp) $
    else val = fltarr(tmp)
    months = ['jan','feb','mar','apr','may','jun','jul','aug','sep','oct','nov','dec']
    if n_elements(zerostring) ne 0 then string0 = strlowcase(strtrim(zerostring,2))
    if n_elements(onestring) ne 0 then  string1 = strlowcase(strtrim(onestring,2))
    if n_elements(twostring) ne 0 then  string2 = strlowcase(strtrim(twostring,2))
    on_ioerror,error
    for i=0L,count-1 do begin
        flag = 1b
        if keyword_set(date) then begin ;construct a ulon64 number yyyymmddhhmmss
           year = '0000' & mon = '00' & day = '00' & hour = '00' & mint = '00' & secd = '00'
           tmp1 = strsplit(str[i],' .,;-/'+string(9B),/extract,count=count1)
           for j=0,count1-1 do begin
               if strmatch(strmid(tmp1[j],0,1),'[!0-9]') then begin ;start with letter
                  ismon = where(months eq strlowcase(strmid(tmp1[j],0,3)),count2)
                  if count2 eq 1 then mon = strtrim(string(ismon+1),2)
               endif else begin             ;start with numbers
                  count2 = 0
                  if strpos(tmp1[j],':') ge 0 then tmp2 = strsplit(tmp1[j],':',/extract,count=count2)
                  if count2 ne 0 then begin
                     hour = tmp2[0]
                     if count2 gt 1 then mint = tmp2[1]
                     if count2 gt 2 then secd = tmp2[2] 
                  endif else begin
                     if strlen(tmp1[j]) ge 4 then year = tmp1[j] else begin
                        if j+1 lt count1 then begin
                           ismon = where(months eq strlowcase(strmid(tmp1[j+1],0,3)),count2)
                           if count2 eq 1 then begin
                              day = tmp1[j]
                              mon = strtrim(string(ismon+1),2)
                              j = j+1
                           endif  
                        endif
                        if (j-1 ge 0) and (count2 eq 0) and (day eq '00') then begin
                           ismon = where(months eq strlowcase(strmid(tmp1[j-1],0,3)),count2)
                           if count2 eq 1 then day = tmp1[j]
                        endif
                        if count2 eq 0 then begin ;assuming the order mm/dd/yy  
                           tmp3 = fix(tmp1[j])
                           if (mon eq '00') and (tmp3 le 12) and (tmp3 gt 0)      then mon  = tmp1[j] $
                           else if (day eq '00') and (tmp3 le 31) and (tmp3 gt 0) then day  = tmp1[j] $
                           else if year eq '0000'                                 then year = tmp1[j]
                        endif
                     endelse
                  endelse
               endelse
           endfor
           while strmatch(strmid(mon,0,1,/reverse_offset),'[!0-9]')  do mon  = strmid(mon,0,strlen(mon)-1)  ;remove trailing letters if any
           while strmatch(strmid(day,0,1,/reverse_offset),'[!0-9]')  do day  = strmid(day,0,strlen(day)-1)
           while strmatch(strmid(hour,0,1,/reverse_offset),'[!0-9]') do hour = strmid(hour,0,strlen(hour)-1)
           while strmatch(strmid(mint,0,1,/reverse_offset),'[!0-9]') do mint = strmid(mint,0,strlen(mint)-1)
           while strmatch(strmid(secd,0,1,/reverse_offset),'[!0-9]') do secd = strmid(secd,0,strlen(secd)-1)
           while strmatch(strmid(year,0,1,/reverse_offset),'[!0-9]') do year = strmid(year,0,strlen(year)-1)
           if strlen(mon)  eq 1 then mon  = '0'+mon
           if strlen(day)  eq 1 then day  = '0'+day
           if strlen(hour) eq 1 then hour = '0'+hour
           if strlen(mint) eq 1 then mint = '0'+mint
           if strlen(secd) eq 1 then secd = '0'+secd 
           if strlen(year) eq 2 then year = strmid(systime(),3,2,/reverse_offset)+year
           val[i] = ulong64(year+mon+day+hour+mint+secd)
        endif else begin
           ;get rid of extra ., for example, '8.0.1'->'8.01'
           tmp1 = strpos(str[i],'.') & tmp2 = strpos(str[i],'.',/reverse_search)
           while (tmp1 ne tmp2) do begin
                 str[i] = strmid(str[i],0,tmp2)+strmid(str[i],tmp2+1,strlen(str[i])-tmp2-1)
                 tmp1   = strpos(str[i],'.') & tmp2 = strpos(str[i],'.',/reverse_search)
           endwhile
           tmp = str[i]+' '  ;default, +' ' to force error with emptry string ''
           if keyword_set(int)       then val[i] = fix(str[i],type=2) $
           else if keyword_set(long) then val[i] = fix(str[i],type=3) $
           else if keyword_set(byte) then begin
              if i eq 0 then val = fix(str[i],type=1) $
              else val = [val,fix(str[i],type=1)]
           endif else if keyword_set(double) then val[i] = double(tmp) $
           else val[i] = float(tmp)
        endelse
        flag = 0b  ;clear error flag
        error: if flag then begin
               if  keyword_set(int)        then val[i] = 0s $
               else if keyword_set(long)   then val[i] = 0l $
               else if keyword_set(double) then val[i] = !values.d_nan $
               else if keyword_set(byte)   then val[i] = 0b $
               else if keyword_set(date)   then val[i] = 0  $
               else val[i] = !values.f_nan
        endif
        if finite(val[i],/nan) or keyword_set(int) or keyword_set(long) then begin
           tmp = strlowcase(strtrim(str[i],2))
           if n_elements(string0) ne 0 then begin
              ind = where(string0 eq tmp,count1)
              if count1 gt 0 then val[i] = 0
           endif
           if n_elements(string1) ne 0 then begin
              ind = where(string1 eq tmp,count1)
              if count1 gt 0 then val[i] = 1
           endif
           if n_elements(string2) ne 0 then begin
              ind = where(string2 eq tmp,count1)
              if count1 gt 0 then val[i] = 2
           endif
        endif
    endfor
    if keyword_set(byte) then count = n_elements(val)
    return,(count eq 1)?val[0]:val
end